<?php
/**
 * Copyright (c) Web Loupe. All rights reserved.
 * This file is part of software that is released under a proprietary license.
 * You must not copy, modify, distribute, make publicly available, or execute
 * its contents or parts thereof without express permission by the copyright
 * holder, unless otherwise permitted by law.
 */

namespace WeloConsorsFinanz\Components;

use Shopware\Components\DependencyInjection\Container as DIContainer;
use Shopware\Components\Model\ModelManager;
use Shopware\Components\Plugin\ConfigReader;
use Shopware\Models\Payment\Payment;
use Shopware_Components_Config;
use Shopware_Components_Snippet_Manager;
use WeloConsorsFinanz\Bootstrap\Database;

/**
 * Class Configuration
 *
 * @author    Cyprien Nkeneng <cyprien.nkeneng@webloupe.de>
 * @copyright Copyright (c) 2017-2020 WEB LOUPE
 * @package   WeloConsorsFinanz\Components
 * @version   1
 */
class Configuration
{
    /**
     * Plugin namespace
     *
     * @var Shopware_Components_Config
     */
    protected $namespace;
    
    /**
     * @var ConfigReader
     */
    private $configReader;
    
    /**
     * @var ModelManager
     */
    private $modelManager;
    
    /**
     * @var Shopware_Components_Snippet_Manager
     */
    private $snippetManager;
    /**
     * @var DIContainer
     */
    private $container;

    /**
     * Configuration constructor.
     * @param                                      $namespace
     * @param ConfigReader                         $configReader
     * @param ModelManager                         $modelManager
     * @param Shopware_Components_Snippet_Manager  $snippetManager
     * @param DIContainer                          $container
     */
    public function __construct(
        $namespace,
        ConfigReader $configReader,
        ModelManager $modelManager,
        Shopware_Components_Snippet_Manager $snippetManager,
        DIContainer $container
    ) {
        $this->configReader = $configReader;
        $this->modelManager = $modelManager;
        $this->namespace = $namespace;
        $this->snippetManager = $snippetManager;
        $this->container = $container;
    }

    public function isActive()
    {
        return (bool) $this->getPluginConfig('WeloConsorsFinanzActivate');
    }

    /**
     * @return array
     * @throws \Exception
     */
    public function getPluginData()
    {
       return [
           'isActive' => (bool) $this->getPluginConfig('WeloConsorsFinanzActivate'),
           'isSandboxActive' => (bool) ($this->getPluginConfig('WeloConsorsFinanzMode') == 'T'),
           'invoicePaymentMethodId' => $this->getPaymentMethodId(Database::INVOICE_PAYMENT_METHOD_NAME),
           'debitPaymentMethodId' => $this->getPaymentMethodId(Database::DEBIT_PAYMENT_METHOD_NAME),
           'isInvoiceActive' => $this->isPaymentActive(Database::INVOICE_PAYMENT_METHOD_NAME),
           'isDebitActive' => $this->isPaymentActive(Database::DEBIT_PAYMENT_METHOD_NAME),
           'codeNumber' => $this->getPluginConfig('WeloConsorsFinanzCodeNumber'),
           'password' => $this->getPluginConfig('WeloConsorsFinanzPassword'),
           'mode' => $this->getPluginConfig('WeloConsorsFinanzMode'),
           'birthdayType' => $this->getPluginConfig('WeloConsorsFinanzBirthdayType'),
       ];
    }

    private function isPaymentActive($name)
    {
        $payment = $this->modelManager->getRepository(Payment::class)
            ->findOneBy(['name' => $name]);

        if ($payment && $payment instanceof Payment) {
            return (bool)$payment->getActive();
        }

        return false;
    }

    private function getPaymentMethodId($name)
    {
        $payment = $this->modelManager->getRepository(Payment::class)
            ->findOneBy(['name' => $name]);

        if ($payment && $payment instanceof Payment) {
            return $payment->getId();
        }

        return 0;
    }

    /**
     * @param $key
     * @return bool|mixed
     * @throws \Exception
     */
    public function getPluginConfig($key)
    {
        if ('' === $this->namespace) {
            throw new \Exception('Plugin namespace does not exist.');
        }
        $config = $this->getShop()
            ? $this->configReader->getByPluginName($this->namespace, $this->getShop())
            : $this->configReader->getByPluginName($this->namespace);
        return isset($config[$key]) ? $config[$key] : false;
    }

    /**
     * @param        $path
     * @param        $name
     * @param string $defaultValue
     * @return mixed|null
     */
    public function getSnippet($path, $name, $defaultValue = '')
    {
        return $this->snippetManager->getNamespace($path)->get($name, $defaultValue);
    }

    /**
     * {@inheritdoc}
     */
    public function getShop()
    {
        if ($this->container->has('shop')) {
            return $this->container->get('shop');
        }

        return null;
    }
}
