<?php
/**
 * Copyright (c) Web Loupe. All rights reserved.
 * This file is part of software that is released under a proprietary license.
 * You must not copy, modify, distribute, make publicly available, or execute
 * its contents or parts thereof without express permission by the copyright
 * holder, unless otherwise permitted by law.
 */

namespace WeloConsorsFinanz\Components\Consors\Api;

use Shopware\Components\DependencyInjection\Container as DIContainer;
use WeloConsorsFinanz\Components\Consors\Request\BoniCheck2Request;
use WeloConsorsFinanz\Components\Consors\Request\Change2Request;
use WeloConsorsFinanz\Components\Consors\Request\Order2Request;
use WeloConsorsFinanz\Components\Consors\Request\OrderStornoRequest;
use WeloConsorsFinanz\Components\Logger\LoggerService;
use WeloConsorsFinanz\Components\Consors\Services\Caster;

/**
 * Class ConsorsSoapClient
 *
 * @author    Cyprien Nkeneng <cyprien.nkeneng@webloupe.de> - www.webloupe.de
 * @copyright Copyright (c) 2017-2020 WEB LOUPE
 * @package   WeloConsorsFinanz\Components\Consors\Api
 * @link      https://www.webloupe.de
 * @version   1
 */
class ConsorsSoapClient extends \SoapClient
{
    const CIPHERS = 'DHE-RSA-AES256-SHA; DHE-RSA-AES128-SHA; AES256-SHAAES128-SHA';
    const CIPHERS2 = 'ECDHE-RSA-AES256-GCM-SHA384;ECDHE-RSA-AES256-SHA384;DHE-RSA-AES256-GCM-SHA384;DHE-RSA-AES256-SHA256;ECDHE-RSA-AES256-SHA;DHE-RSA-AES256-SHA';

    /**
     * @var string
     */
    private $wsdl;

    /**
     * @var string
     */
    private $cafile;
    /**
     * @var LoggerService
     */
    private $loggerService;
    /**
     * @var mixed
     */
    private $pluginRoot;

    /**
     * Reference of class caster service
     */
    private $caster = null;

    /**
     * @param DIContainer $container
     * @throws \SoapFault
     * @throws \Exception
     */
    public function __construct(DIContainer $container) {
        $this->pluginRoot = $container->getParameter('welo_consors_finanz.plugin_dir');
        $this->loggerService = $container->get('welo_consors_finanz.components.logger.logger_service');

        $this->caster = new Caster();

        $this->setApiFiles();
        parent::__construct($this->wsdl, $this->getApiOptions());
    }

    /**
     *
     * @param
     *            $method
     * @param BoniCheck2Request $params
     * @return mixed
     * @throws ConsorsSoapClientFault
     */
    public function callClient($method, BoniCheck2Request $params)
    {
//        $params->setHeader(new HeaderRequest($this->config));
//        $this->dumb($params);
        return $this->_call($method, [$params]);
    }

    /**
     *
     * @param BoniCheck2Request $request
     * @return mixed
     * @throws ConsorsSoapClientFault
     */
    public function boniCheck2(BoniCheck2Request $request)
    {
        return $this->_call('BoniCheck2', [$request]);
    }

    /**
     *
     * @param Order2Request $request
     * @return mixed
     * @throws ConsorsSoapClientFault
     */
    public function order2(Order2Request $request)
    {
        return $this->_call('Order2', [$request]);
    }

    /**
     *
     * @param OrderStornoRequest $request
     * @return mixed
     * @throws ConsorsSoapClientFault
     */
    public function cancelOrder(OrderStornoRequest $request)
    {
        return $this->_call('OrderStorno', [$request]);
    }

    /**
     *
     * @param Change2Request $request
     * @return mixed
     * @throws ConsorsSoapClientFault
     */
    public function changeOrder(Change2Request $request)
    {
        return $this->_call('Change2', [$request]);
    }

    /**
     * Make SOAP call and catch faults
     *
     * @param string $function
     * @param array $params
     * @return mixed
     * @throws ConsorsSoapClientFault
     */
    private function _call($function, $params)
    {
        try {
            return $this->caster->cast($function.'Response',
                $this->__soapCall($function, $params));

        } catch (ConsorsSoapClientFault $e) {
            $this->loggerService->error($e->getMessage());
            throw new ConsorsSoapClientFault($e->getCode(), $e->getMessage());
        }
    }

    private function getApiOptions()
    {
        $sslContextOptions = [
            'ssl' => [
                'verify_peer' => true,
//                'allow_self_signed' => true,
                'cafile' => $this->cafile,
                'ciphers' => self::CIPHERS2
            ]
        ];

        $sslContext = stream_context_create($sslContextOptions);

        return [
            'stream_context' => $sslContext,
            'soap_version' => SOAP_1_1,
            'trace' => 1,
            'exceptions' => 1
        ];
    }

    private function setApiFiles()
    {
        if (!class_exists('SOAPClient')) {
            throw new \Exception('Dieses Plugin benötigt die PHP-Erweiterung "SOAPClient". Bitte wenden Sie sich an Ihren Hoster bzw. Serveradministrator, um diese zu aktivieren.');
        }

        $this->wsdl = $this->pluginRoot . '/wsdl/service_1_4_1.wsdl';
        $this->cafile = $this->pluginRoot . '/wsdl/00_real_solution_ca_bundle.pem';

        if (!file_exists($this->wsdl)) {
            $this->loggerService->error(sprintf('The wsdl %s is missing', $this->wsdl));
        }

        if (!file_exists($this->cafile)) {
            $this->loggerService->error(sprintf('The bundle %s is missing', $this->cafile));
        }

        ini_set('soap.wsdl_cache_enabled',0);
        ini_set('soap.wsdl_cache_ttl',0);
        libxml_disable_entity_loader(false);
    }

    function dumb($data){
        highlight_string("<?php\n " . var_export($data, true) . "?>");
        echo '<script>document.getElementsByTagName("code")[0].getElementsByTagName("span")[1].remove() ;document.getElementsByTagName("code")[0].getElementsByTagName("span")[document.getElementsByTagName("code")[0].getElementsByTagName("span").length - 1].remove() ; </script>';
        die();
    }
}
