<?php
/**
 * Copyright (c) Web Loupe. All rights reserved.
 * This file is part of software that is released under a proprietary license.
 * You must not copy, modify, distribute, make publicly available, or execute
 * its contents or parts thereof without express permission by the copyright
 * holder, unless otherwise permitted by law.
 */

namespace WeloConsorsFinanz\Components\Consors\Services;

use Shopware\Components\DependencyInjection\Container as DIContainer;
use Shopware\Components\Model\ModelManager;
use WeloConsorsFinanz\Components\Configuration;
use WeloConsorsFinanz\Components\Consors\Api\ConsorsSoapClient;
use WeloConsorsFinanz\Components\DependencyProvider;
use WeloConsorsFinanz\Components\Logger\LoggerService;
use WeloConsorsFinanz\Components\Services\ConsorsService;

/**
 * Class AbstractService
 *
 * @author    Cyprien Nkeneng <cyprien.nkeneng@webloupe.de> - www.webloupe.de
 * @copyright Copyright (c) 2017-2020 WEB LOUPE
 * @package   WeloConsorsFinanz\Components\Consors\Services
 * @link      https://www.webloupe.de
 * @version   1
 */
abstract class AbstractService
{
    protected $configData;

    /** @var ConsorsSoapClient */
    protected $consorsApi;
    /**
     * @var DependencyProvider
     */
    protected $dependencyProvider;

    protected $orderData;

    private $listOfChars = [
        'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'J', 'K', 'L', 'M',
        'N', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z',
        'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l',
        'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x',
        'y', 'z', '1', '2', '3', '4', '5', '6', '7', '8', '9'
    ];
    /**
     * @var DIContainer
     */
    protected $container;
    /**
     * @var ModelManager
     */
    protected $modelManager;
    /**
     * @var LoggerService
     */
    protected $loggerService;
    /**
     * @var ConsorsService
     */
    protected $consorsService;

    /**
     * BoniCheck2Service constructor.
     * @param Configuration      $configuration
     * @param DIContainer        $container
     * @param DependencyProvider $dependencyProvider
     * @param ModelManager       $modelManager
     * @throws \SoapFault
     */
    public function __construct(
        Configuration $configuration,
        DIContainer $container,
        DependencyProvider $dependencyProvider,
        ModelManager $modelManager
    ) {
        $this->configData = $configuration->getPluginData();
        $this->dependencyProvider = $dependencyProvider;

        $this->consorsApi = new ConsorsSoapClient($container);

        $session = $this->dependencyProvider->getSession();
        $this->orderData = $session['sOrderVariables'];
        $this->container = $container;
        $this->modelManager = $modelManager;
        $this->loggerService = $container->get('welo_consors_finanz.components.logger.logger_service');
        $this->consorsService = $container->get('welo_consors_finanz.components.services.consors_service');
    }

    /**
     * @param $changetyp
     * @return array
     */
    protected function getDefaultParams($changetyp)
    {
        return [
            'config' => $this->configData,
            'anfrageart' => $changetyp,
            'terminalclientaddress' => $_SERVER["REMOTE_ADDR"],
            'userreference' => $this->getUserReference(),
            'bestellwert' => $this->orderData['sBasket']['sAmount'] * 100,
        ];
    }

    protected function getUserReference()
    {
        $user = $this->orderData['sUserData']['additional']['user'];
        $consorsUserReference = $this->modelManager->getConnection()->createQueryBuilder()
            ->select('consors_finanz_userreference')
            ->from('s_user_attributes', 'attribute')
            ->leftJoin('attribute','s_user', 'user', 'attribute.userID = user.id')
            ->where('user.id = :id')
            ->setParameter('id', (int)$user['id'])
            ->execute()->fetchColumn();

        if ('' == $consorsUserReference) {
            $consorsUserReference = $this->consorsService->getNewUserReference($user['id']);
        }
        return $consorsUserReference;
    }

    /**
     * @param $consorsUserReference
     * @return bool
     */
    private function existUserReference($consorsUserReference) {
        return (bool) $this->modelManager->getConnection()->createQueryBuilder()
            ->select('*')
            ->from('s_user_attributes')
            ->where('consors_finanz_userreference = :consors_finanz_userreference')
            ->setParameter('consors_finanz_userreference', $consorsUserReference)
            ->execute()
            ->fetch();
    }

    /**
     * This method generates a random string. The $length indicates
     * the length of the resulting string. The $charsToExclude indicates
     * which of the elements are not used. By default 'l', 'i' and '1' are
     * not used.
     *
     * @param integer $length
     * @param string $charsToExclude
     * @return string Random string
     */
    private function getRandomString($length, $charsToExclude = 'li1' ) {
        $characters = [];
        foreach ($this->listOfChars as $char) {
            if (!strstr($charsToExclude, $char)) {
                $characters[] = $char;
            }
        }

        $numberOfCharacters = count($characters);
        $code = '';
        for ($c = 0; $c < $length; $c++) {
            $randIndex = mt_rand(0, $numberOfCharacters-1);
            $code .= $characters[$randIndex];
        }

        return $code;
    }

    protected function getAddress($address)
    {
        return [
            'strasse' => $address['street'],
            'plz' => ($address['zipcode'] != null) ? $address['zipcode'] : $address['zipCode'],
            'ort' => $address['city'],
            'land' => $this->getCountryIso($address['countryId']),
        ];
    }

    /**
     * @return array
     */
    public function getBasket()
    {
        $basket = $this->dependencyProvider->getBasket()->sGetBasket();

        if (!Shopware()->System()->sUSERGROUPDATA['tax'] && Shopware()->System()->sUSERGROUPDATA['id']) {
            $basket['sTaxRates'] = $this->getTaxRates($basket);
            $basket['sAmount'] = round($basket['AmountNetNumeric'], 2);
            $basket['sAmountTax'] = round($basket['AmountWithTaxNumeric'] - $basket['AmountNetNumeric'], 2);
            $basket['sAmountWithTax'] = round($basket['AmountWithTaxNumeric'], 2);
        } else {
            $basket['sTaxRates'] = $this->getTaxRates($basket);
            $basket['sAmount'] = $basket['AmountNumeric'];
            $basket['sAmountTax'] = round($basket['AmountNumeric'] - $basket['AmountNetNumeric'], 2);
        }

        return $basket;
    }

    /**
     *
     * @param array $basket array returned from this->getBasket
     *
     * @return array
     */
    public function getTaxRates($basket)
    {
        $result = [];

        if (!empty($basket['sShippingcostsTax'])) {
            if (!empty($basket['sShippingcostsTaxProportional'])) {
                /** @var \Shopware\Components\Cart\Struct\Price $shippingTax */
                foreach ($basket['sShippingcostsTaxProportional'] as $shippingTax) {
                    $result[number_format($shippingTax->getTaxRate(), 2)] += $shippingTax->getTax();
                }
            } else {
                $basket['sShippingcostsTax'] = number_format((float) $basket['sShippingcostsTax'], 2);

                $result[$basket['sShippingcostsTax']] = $basket['sShippingcostsWithTax'] - $basket['sShippingcostsNet'];
                if (empty($result[$basket['sShippingcostsTax']])) {
                    unset($result[$basket['sShippingcostsTax']]);
                }
            }
        }

        if (empty($basket['content'])) {
            ksort($result, SORT_NUMERIC);

            return $result;
        }

        foreach ($basket['content'] as $item) {
            if (!empty($item['tax_rate'])) {
            } elseif (!empty($item['taxPercent'])) {
                $item['tax_rate'] = $item['taxPercent'];
            } elseif ($item['modus'] == 2) {
                // Ticket 4842 - dynamic tax-rates
                $resultVoucherTaxMode = Shopware()->Db()->fetchOne(
                    'SELECT taxconfig FROM s_emarketing_vouchers WHERE ordercode=?',
                    [$item['ordernumber']]
                );

                $tax = null;

                // Old behaviour
                if (empty($resultVoucherTaxMode) || $resultVoucherTaxMode === 'default') {
                    $tax = Shopware()->Config()->get('sVOUCHERTAX');
                } elseif ($resultVoucherTaxMode === 'auto') {
                    // Automatically determinate tax
                    $tax = $this->dependencyProvider->getBasket()->getMaxTax();
                } elseif ($resultVoucherTaxMode === 'none') {
                    // No tax
                    $tax = '0';
                } elseif ((int) $resultVoucherTaxMode) {
                    // Fix defined tax
                    $tax = Shopware()->Db()->fetchOne('
                    SELECT tax FROM s_core_tax WHERE id = ?
                    ', [$resultVoucherTaxMode]);
                }
                $item['tax_rate'] = $tax;
            } else {
                // Ticket 4842 - dynamic tax-rates
                $taxAutoMode = Shopware()->Config()->get('sTAXAUTOMODE');
                if (!empty($taxAutoMode)) {
                    $tax = $this->dependencyProvider->getBasket()->getMaxTax();
                } else {
                    $tax = Shopware()->Config()->get('sDISCOUNTTAX');
                }
                $item['tax_rate'] = $tax;
            }

            if (empty($item['tax_rate']) || empty($item['tax'])) {
                continue;
            } // Ignore 0 % tax

            $taxKey = number_format((float) $item['tax_rate'], 2);

            $result[$taxKey] += str_replace(',', '.', $item['tax']);
        }

        ksort($result, SORT_NUMERIC);

        return $result;
    }

    /**
     * @param int $countryId
     * @return bool|string
     */
    protected function getCountryIso($countryId = 0)
    {
        return $this->modelManager->getConnection()->createQueryBuilder()
            ->select('countryiso')
            ->from('s_core_countries')
            ->where('id = :id')
            ->setParameter('id', $countryId)
            ->setMaxResults(1)
            ->execute()
            ->fetchColumn();
    }

    protected function getUserBankData($userId)
    {
        return $this->modelManager->getConnection()->createQueryBuilder()
            ->select('*')
            ->from('s_core_payment_data')
            ->where('user_id = :id')
            ->setParameter('id', $userId)
            ->execute()
            ->fetch();
    }

    protected function translateSalutation($salutation)
    {
        $result = '';
        switch ($salutation) {
            case 'mr':
                $result = 'Herr';
                break;
            case 'ms':
                $result = 'Frau';
                break;
        }
        return $result;
    }

    function dumb($data){
        highlight_string("<?php\n " . var_export($data, true) . "?>");
        echo '<script>document.getElementsByTagName("code")[0].getElementsByTagName("span")[1].remove() ;document.getElementsByTagName("code")[0].getElementsByTagName("span")[document.getElementsByTagName("code")[0].getElementsByTagName("span").length - 1].remove() ; </script>';
        die();
    }
}