<?php
/**
 * Copyright (c) Web Loupe. All rights reserved.
 * This file is part of software that is released under a proprietary license.
 * You must not copy, modify, distribute, make publicly available, or execute
 * its contents or parts thereof without express permission by the copyright
 * holder, unless otherwise permitted by law.
 */

namespace WeloConsorsFinanz\Components\Consors\Services;

use Shopware\Models\Order\Order;
use WeloConsorsFinanz\Components\Consors\Api\ConsorsSoapClientFault;
use WeloConsorsFinanz\Components\Consors\Request\Change2Request;
use WeloConsorsFinanz\Components\Consors\Response\Change2Response;
use WeloConsorsFinanz\Components\SettingsTable;

/**
 * Class ChangeService
 *
 * @author    Cyprien Nkeneng <cyprien.nkeneng@webloupe.de> - www.webloupe.de
 * @copyright Copyright (c) 2017-2020 WEB LOUPE
 * @package   WeloConsorsFinanz\Components\Consors\Services
 * @link      https://www.webloupe.de
 * @version   1
 */
class ChangeService extends AbstractService
{
    /**
     * @param array $document
     * @param       $changetyp
     * @param       $isGutschrift
     * @return Change2Response
     * @throws ConsorsSoapClientFault
     */
    public function change(array $document, $changetyp, $isGutschrift = false)
    {
//        $this->dumb($document);
        $request = $this->getParams($document, $changetyp, $isGutschrift);

//        if (20085 == $document['order']['number']) {
//            $this->dumb($request);
//        }

        /** @var Change2Response $response */
        $response = $this->consorsApi->changeOrder($request);
        return $response;
    }

    /**
     * @param array $document
     * @param       $changetyp
     * @param bool  $isGutschrift
     * @return Change2Request
     */
    private function getParams(array $document, $changetyp, $isGutschrift = false)
    {
        $params = array_merge(
            $this->getDefaultParams($changetyp),
            $this->getChangeParams($document, $isGutschrift)
        );

        return new Change2Request($params);
    }

    /**
     * @param $changetyp
     * @return array
     */
    protected function getDefaultParams($changetyp)
    {
        return [
            'config' => $this->configData,
            'changetyp' => $changetyp
        ];
    }

    /**
     * @param array $document
     * @param bool  $isGutschrift
     * @return \string[][]
     */
    private function getChangeParams(array $document, $isGutschrift = false)
    {
        $belege = $this->getBelege($document, $isGutschrift);
        $debitor = $this->getDebitor($document);

        return [
            'referenzen' => [
                'referenztyp' => 'bestellnummer',
                'referenzwert' => (true == $isGutschrift) ? $document['documentId'] : $document['order']['number'],
            ],
            'debitor' => $debitor,
            'belege' => $belege,
        ];
    }

    private function getDebitor($orderData)
    {
        $billing = $orderData['order']['billing'];
        $customer = $orderData['order']['customer'];
        return [
            'kundennummer' => $customer['number'],
            'bezeichnung' => $customer['firstname'] . ' ' . $customer['lastname'],
            'person' => $this->getPerson($customer, $billing)
        ];
    }

    private function getBelege($orderData, $isGutschrift = false)
    {
        $order = $orderData['order'];
        $customer = $order['customer'];
        $payment = $order['payment'];

        return [
            'kundennummer' => $customer['number'],
            'belegtyp' => (true == $isGutschrift) ? SettingsTable::BELEG_TYPE_GUTSCHRIFT : SettingsTable::BELEG_TYPE_RECHNUNG,
            'belegnummer' => $orderData['documentId'],
            'belegdatum' => $orderData['date']->format('Y-m-d'),
            'belegreferenz' => $orderData['documentId'],
            'faellig' => date('Y-m-d', strtotime('+30 days',strtotime($orderData['date']->format('Y-m-d')))),
            'waehrung' => $order['currency'],
            'betrag' => (true == $isGutschrift) ? $orderData['amount'] * -1 : $orderData['amount'],
            'betrag_netto' => $orderData['amount'],
            'zahlart' => ('WeloConsorsFinanzDebit' == $payment['name']) ? 'LA' : 'UEB',
            'rgbezahlt' => 'NEIN',
            'belegtext' => ('WeloConsorsFinanzDebit' == $payment['name']) ? 'Lastschrift' : 'Ueberweisung',
            'belegpositionen' => $this->getBelegPositions($orderData['order']),
//            'lieferperson' => $this->getPerson($customer, $shipping, true),
//            'lieferanschrift' => $this->getAddress($shipping),
        ];
    }

    private function getBelegPositions($orderData)
    {
        $customer = $orderData['customer'];
        $details = $orderData['details'];

        $result = $articles = [];
        $taxRate = 0;
        foreach ($details as $detail) {
            $taxRate = $detail['taxRate'];
            $articles[] = [
                'menge' => $detail['quantity'],
                'einheit' => ($detail['unit'] != 'Stück') ? $detail['unit'] : '',
                'artikelnummer' => $detail['articleNumber'],
                'artikelbezeichnung' => $detail['articleName'],
                'artikelzusatz' => '',
                'betrag' => $detail['price'],
                'betrag_netto' => $detail['price'],
                'umsatzsteuersatz' => $detail['taxRate'],
                'warengruppe' => '',
            ];
        }

        $result[] = [
            'kundennummer' => $customer['number'],
            'belegpositionsnummer' => $orderData['number'],
            'betrag' => $orderData['invoiceAmount'],
            'betrag_netto' => $orderData['invoiceAmountNet'],
            'umsatzsteuersatz' => $taxRate,
            'waehrung' => $orderData['currency'],
            'artikel' => $articles
        ];

        return $result;
    }

    private function getPerson($user, $address, $liefer = false)
    {
        $firstName = ($address['firstname'] != null) ? $address['firstname'] : $address['firstName'];
        $lastName = ($address['lastname'] != null) ? $address['lastname'] : $address['lastName'];

        $person = [
            'salutation' => $this->translateSalutation($address['salutation']),
            'vorname' => ($liefer === true) ? $firstName : $user['firstname'],
            'name' => ($liefer === true) ? $lastName : $user['lastname'],
            'geburtsdatum' => (is_string($user['birthday'])) ? $user['birthday'] : $user['birthday']->format('Y-m-d'),
            'hauptanschrift' => [
                'adresse' => $this->getAddress($address),
                'zusatz' => $address['additionalAddressLine1']
            ],
            'kontakte' => [
                [
                    'typ' => 'mobil',
                    'kontakt' => $address['phone']
                ],
                [
                    'typ' => 'email',
                    'kontakt' => $user['email']
                ]
            ],
            'rechtsform' => '',
            'handelsregisternummer' => '',
            'amtsgericht' => '',
        ];

        $bankData = $this->getUserBankData($user['id']);
        if (!empty($bankData)) {
            $person = array_merge(
                $person,
                ['bankverbindungen' => $bankData]
            );
        }
        return $person;
    }
}