<?php
/**
 * Copyright (c) Web Loupe. All rights reserved.
 * This file is part of software that is released under a proprietary license.
 * You must not copy, modify, distribute, make publicly available, or execute
 * its contents or parts thereof without express permission by the copyright
 * holder, unless otherwise permitted by law.
 */

namespace WeloConsorsFinanz\Components\PaymentMethods;

use Doctrine\ORM\AbstractQuery;
use WeloConsorsFinanz\Bootstrap\Database;

/**
 * Class Debit
 *
 * @author    Cyprien Nkeneng <cyprien.nkeneng@webloupe.de> - www.webloupe.de
 * @copyright Copyright (c) 2017-2020 WEB LOUPE
 * @package   WeloConsorsFinanz\Components\PaymentMethods
 * @link      https://www.webloupe.de
 * @version   1
 */
class Debit extends AbstractPaymentMethod
{
    /**
     * {@inheritdoc}
     */
    public function validate($paymentData)
    {
        $birthdayType = $this->pluginData['WeloConsorsFinanzBirthdayType'];

        $sErrorFlag = [];
        $fields = [
            'ConsorsFinanzDebitPhone',
            'ConsorsFinanzDebitIban'
        ];

        if ($birthdayType == 'direct') {
            $fields[2]['ConsorsFinanzDebitBirthday'] = [
                'day',
                'month',
                'year'
            ];
        } else {
            $fields[2] = 'ConsorsFinanzDebitBirthday';
        }

        foreach ($fields as $field) {
            $value = $paymentData[$field] ?: '';
            $value = trim($value);

            if (empty($value)) {
                $sErrorFlag[$field] = true;
            }

            if (is_array($field)) {
                foreach ($field as $key => $date) {
                    foreach ($date as $item) {
                        $value = $paymentData[$key][$item] ?: '';
                        $value = trim($value);
                        if (empty($value)) {

                            $sErrorFlag[$field][$item] = true;
                        }
                    }
                }
            }
        }

        if (count($sErrorFlag) > 0) {
        $sErrorMessages[] = Shopware()->Snippets()->getNamespace('frontend/account/internalMessages')
            ->get('ErrorFillIn', 'Please fill in all red fields');

        return [
                'sErrorFlag' => $sErrorFlag,
                'sErrorMessages' => $sErrorMessages,
            ];
        }

        return [];
    }

    /**
     * {@inheritdoc}
     */
    public function savePaymentData($userId, \Enlight_Controller_Request_Request $request)
    {
        if (0 == (int)$userId) {
            return;
        }

        $this->saveUserBirthday($userId, $request);
        $this->saveUserPhone($userId, $request);

        $session = Shopware()->Session();
        $agb = ($request->getParam('ConsorsFinanzDebitAgb') === 'true' ? 1 : 0);
        $session->offsetSet('consorsAgbChecked', $agb);

        $lastPayment = $this->getCurrentPaymentDataAsArray($userId);

        $paymentMean = Shopware()->Models()->getRepository('\Shopware\Models\Payment\Payment')->
        getActivePaymentsQuery(['name' => Database::DEBIT_PAYMENT_METHOD_NAME])->getOneOrNullResult(AbstractQuery::HYDRATE_ARRAY);

        $data = [
            'iban' => $request->getParam('ConsorsFinanzDebitIban'),
            'bic' => $request->getParam('ConsorsFinanzDebitBic'),
//            'bankname' => $request->getParam('ConsorsFinanzDebitBankName'),
//            'account_holder' => $request->getParam('ConsorsFinanzDebitAccountHolder'),
        ];

        if (!$lastPayment) {
            $date = new \DateTime();
            $data['created_at'] = $date->format('Y-m-d');
            $data['payment_mean_id'] = $paymentMean['id'];
            $data['user_id'] = $userId;
            Shopware()->Db()->insert('s_core_payment_data', $data);
        } else {
            $where = [
                'payment_mean_id = ?' => $paymentMean['id'],
                'user_id = ?' => $userId,
            ];

            Shopware()->Db()->update('s_core_payment_data', $data, $where);
        }
    }

    /**
     * {@inheritdoc}
     */
    public function getCurrentPaymentDataAsArray($userId)
    {
        $paymentData = Shopware()->Models()->getRepository('\Shopware\Models\Customer\PaymentData')
            ->getCurrentPaymentDataQueryBuilder($userId, Database::DEBIT_PAYMENT_METHOD_NAME)->getQuery()->getOneOrNullResult(AbstractQuery::HYDRATE_ARRAY);

        if (isset($paymentData)) {
            return [
                'ConsorsFinanzDebitBirthday' => $this->getUserBirthday($userId),
                'ConsorsFinanzDebitPhone' => $this->getUserPhone($userId),
                'ConsorsFinanzDebitIban' => $paymentData['iban'],
                'ConsorsFinanzDebitBic' => $paymentData['bic'],
//                'ConsorsFinanzDebitBankName' => $paymentData['bankName'],
//                'ConsorsFinanzDebitAccountHolder' => $paymentData['accountHolder'],
            ];
        }
    }

    /**
     * {@inheritdoc}
     */
    public function createPaymentInstance($orderId, $userId, $paymentId)
    {
        $orderAmount = Shopware()->Models()->createQueryBuilder()
            ->select('orders.invoiceAmount')
            ->from('Shopware\Models\Order\Order', 'orders')
            ->where('orders.id = ?1')
            ->setParameter(1, $orderId)
            ->getQuery()
            ->getSingleScalarResult();

        $addressData = Shopware()->Models()->getRepository('Shopware\Models\Customer\Customer')
            ->find($userId)->getDefaultBillingAddress();

        $debitData = $this->getCurrentPaymentDataAsArray($userId);

        $date = new \DateTime();
        $data = [
            'payment_mean_id' => $paymentId,
            'order_id' => $orderId,
            'user_id' => $userId,
            'firstname' => $addressData->getFirstname(),
            'lastname' => $addressData->getLastname(),
            'address' => $addressData->getStreet(),
            'zipcode' => $addressData->getZipcode(),
            'city' => $addressData->getCity(),
            'iban' => $debitData['ConsorsFinanzDebitIban'],
            'bic' => $debitData['ConsorsFinanzDebitBic'],
            'bank_name' => $debitData['ConsorsFinanzDebitBankName'],
            'account_holder' => $debitData['ConsorsFinanzDebitAccountHolder'],
            'amount' => $orderAmount,
            'created_at' => $date->format('Y-m-d'),
        ];

        Shopware()->Db()->insert('s_core_payment_instance', $data);

        return true;
    }

    private function saveUserPhone($userId, \Enlight_Controller_Request_Request $request)
    {
        $phone = $request->getParam('ConsorsFinanzDebitPhone');
        $this->savePhone($userId, $phone);
    }

    private function saveUserBirthday($userId, \Enlight_Controller_Request_Request $request)
    {
        $birthday = $request->getParam('ConsorsFinanzDebitBirthdayHidden');
        if (empty($birthday)) {
            $this->saveDate(
                $userId,
                $request->getParam('ConsorsFinanzDebitBirthday'),
                $request->getParam('ConsorsFinanzDebitBirthdayType')
            );
        }
    }

    function dumb($data) {
        highlight_string("<?php\n " . var_export($data, true) . "?>");
        echo '<script>document.getElementsByTagName("code")[0].getElementsByTagName("span")[1].remove() ;document.getElementsByTagName("code")[0].getElementsByTagName("span")[document.getElementsByTagName("code")[0].getElementsByTagName("span").length - 1].remove() ; </script>';
        die();
    }
}