;(function($, window) {
    'use strict';

    $.plugin('ConsorsFinanzDebit', {
        defaults: {
            /**
             * @string _name
             */
            _name: 'consorsFinanzDebit',

            hiddenClass: 'is--hidden',

            submitButton: 'submit',

            agbInput: '.consors-finanz-debit-agb',

            bicInput: '.consors-finanz-debit-bic',

            agbLabel: '.consors-finanz-debit-agb-label',

            url: '',

            /**
             * An object containing the names of all fields that are currently invalid and their resepctive error messages.
             */
            invalidFields: {},

            /**
             * The currency to use for creating Stripe SEPA sources.
             */
            currency: null,

            /**
             * The snippets used for Stripe error descriptions.
             */
            snippets: {
                error: {
                    title: 'Error',
                },
            },
        },

        /** @object - Error status values */
        _status: {
            success: 'success',
            undefined: 'undefined',
        },

    init: function () {
        var me = this;

        me.applyDataAttributes();
        me.registerEvents();
        $.publish('plugin/ConsorsFinanzDebitSubmitForm/init', me);
    },

    /**
     * @private
     * @method registerEvents
     */
    registerEvents: function () {
        var me = this;

        me.$bicInput = me.$el.find(me.opts.bicInput);
        me.setupForm();

        // Add listener on changes of the selected payment method to setup the form again
        $.subscribe('plugin/swShippingPayment/onInputChanged', function () {
            me.setupForm();
        });
    },

    setupForm: function () {
        var me = this;

        me.formEl('*.consors-finanz-debit-agb-label ').swModalbox();
        me.formEl('*.datepicker').swDatePicker();

        this.$body = this.$el;
        this.invalidFields = {};
        this.updateValidationErrors();

        if (this.getActiveConsorsFinanzDebitForm()) {
            $('.consors-finanz-debit-form').show();
            $('#debit-birthday_day').addClass('is--required');
            $('#debit-birthday_month').addClass('is--required');
            $('#debit-birthday_year').addClass('is--required');

            $('.consors-finanz-invoice-form').hide();
            $('#invoice-birthday_day').removeClass('is--required').removeAttr('required');
            $('#invoice-birthday_month').removeClass('is--required').removeAttr('required');
            $('#invoice-birthday_year').removeClass('is--required').removeAttr('required');

            this.observeForm();
        }
    },

    observeForm: function () {
        var me = this;

        me.formEl('input[class^="consors-finanz-"]').each(function () {
            // Save the current value and add listener
            var elem = $(this);
            elem.data('oldVal', elem.val());
            elem.on('propertychange keyup input paste change', {scope: me}, me.onFieldChange);
        });

        $("#shippingPaymentForm").click(function(){
            me.findForm().one('submit', { scope: me }, me.onFormSubmission);
        });
    },

    onFormSubmission: function (event) {
        var me = event.data.scope,
            form = $(this);

        me.resetErrorDiv();

        // Prevent the form from being submitted until the card details are validated and the customer is approved
        event.preventDefault();

        // Trigger the validation of all fields
        me.formEl('input[class^="consors-finanz-"]').each(function () {
            $(this).trigger('propertychange', [ true ]);
        });

        // Check for invalid fields
        if (Object.keys(me.invalidFields).length > 0) {
            return;
        }

        me.findForm().submit();
    },

    markFieldInvalid: function (fieldId, errorCode, message) {
        this.invalidFields[fieldId] = this.opts.snippets.error[errorCode || ''] || message || 'Unknown error';
        this.updateValidationErrors();
    },

    /**
     * Removes all validation errors for the field with the given 'fieldId' and triggers an update of the displayed
     * validation errors.
     *
     * @param String fieldId
     */
    markFieldValid: function (fieldId) {
        delete this.invalidFields[fieldId];
        this.updateValidationErrors();
    },

    /**
     * Validates the field value to not be empty. If the validation failes, the field is marked invalid.
     *
     * @param Event event
     * @param boolean|undefined force
     */
    onFieldChange: function (event, force) {
        var me = event.data.scope,
            elem = $(this),
            name = elem.attr('name'),
            placeholder = elem.attr('placeholder'),
            $abgLabel = $(me.opts.agbLabel);

        if ('ConsorsFinanzDebitIban' === name) {
            if (elem.val().indexOf("DE") >= 0) {
                $(me.opts.bicInput).removeClass('instyle_error has--error');
                me.markFieldValid('debit-bic');
            } else {
                $(me.opts.bicInput).addClass('instyle_error has--error');
                me.markFieldInvalid('debit-bic', ('invalid_' + 'debit-bic'), placeholder + ' darf nich leer sein!');
            }
        }

        if ('ConsorsFinanzDebitBic' !== name) {
            if ('debit-agb' !== name) {
                // Check if value has changed, if 'force' is not set
                if (!force && elem.data('oldVal') === elem.val()) {
                    return;
                }

                elem.data('oldVal', elem.val());

                // Validate the field
                if (elem.val().trim().length === 0) {
                    elem.addClass('instyle_error has--error');
                    me.markFieldInvalid(name, ('invalid_' + name), placeholder + ' darf nich leer sein!');
                } else {
                    elem.removeClass('instyle_error has--error');
                    me.markFieldValid(name);
                }
            } else {
                if (!this.checked) {
                    elem.addClass('instyle_error has--error');
                    $abgLabel.addClass('has--error');
                    me.markFieldInvalid(name, ('invalid_' + name), 'Sie müssen die AGB akzeptieren!');
                } else {
                    elem.removeClass('instyle_error has--error');
                    $abgLabel.removeClass('has--error');
                    me.markFieldValid(name);
                }
            }
        }
    },

    /**
     * Tries to find a consors finanz form for the currently active payment method. That is, if a consors finanz payment
     * method is selected, its form is returned, otherwise returns null.
     *
     * @return jQuery|null
     */
    getActiveConsorsFinanzDebitForm: function () {
        var form = $('input[id^="payment_mean"]:checked').closest('.payment--method').find('.consors-finanz-debit-form');

        return (form.length > 0) ? form.first() : null;
    },

    /**
     * Sets the given message in the general error box and scrolls the page to make it visible.
     *
     * @param String message A Stripe error message.
     */
    handleError: function (message) {
        // Display the error information above the credit card form and scroll to its position
        this.formEl('.consors-finanz-error-box').show().children('.error-content').html(message);
        $('body').animate({
            scrollTop: (this.getActiveConsorsFinanzDebitForm().offset().top - 100)
        }, 500);
    },

    /**
     * Sets the given message in the general error box and scrolls the page to make it visible.
     *
     * @param String message A Stripe error message.
     */
    resetErrorDiv: function() {
        this.formEl('.consors-finanz-error-box').hide();
    },

    /**
     * Finds both submit buttons on the page and adds the 'disabled' attribute as well as the loading indicator to each
     * of them.
     */
    setSubmitButtonsLoading: function () {
        // Reset the button first to prevent it from being added multiple loading indicators
        this.resetSubmitButtons();
        $('#shippingPaymentForm button[type="submit"], .confirm--actions button[form="shippingPaymentForm"]').each(function () {
            $(this).html($(this).text() + '<div class="js--loading"></div>').attr('disabled', 'disabled');
        });
    },

    /**
     * Finds both submit buttons on the page and resets them by removing the 'disabled' attribute as well as the
     * loading indicator.
     */
    resetSubmitButtons: function () {
        $('#shippingPaymentForm button[type="submit"], .confirm--actions button[form="shippingPaymentForm"]').each(function () {
            $(this).removeAttr('disabled').find('.js--loading').remove();
        });
    },

    updateValidationErrors: function () {
        var me = this,
            errorBox = me.formEl('.consors-finanz-validation-error-box'),
            boxContent = errorBox.find('.error-content');
        boxContent.empty();
        if (Object.keys(me.invalidFields).length > 0) {
            // Update the error box message and make it visible
            var listEl = $('<ul></ul>')
                .addClass('alert--list')
                .appendTo(boxContent);
            Object.keys(me.invalidFields).forEach(function (key) {
                var row = $('<li></li>')
                    .addClass('list--entry')
                    .text(me.invalidFields[key])
                    .appendTo(listEl);
            });
            errorBox.show();
        } else {
            errorBox.hide();
        }
    },

    /**
     * add namespace for events
     * @param event
     * @returns {string}
     *
     */
    getEventName: function(event) {
        var me = this;

        return event + '.' + me._name;
    },

    formEl: function (selector) {
        var form = this.getActiveConsorsFinanzDebitForm();
        return (form) ? form.find(selector) : $('consors_finanz_debit_not_found');
    },

    /**
     * @return jQuery The main payment selection form element.
     */
    findForm: function () {
        return $('#shippingPaymentForm');
    },

    destroy: function () {
        this._destroy();
    }

    });
    StateManager.addPlugin('*[data-consors-finanz-debit="true"]', 'ConsorsFinanzDebit');
})(jQuery, window);
