<?php

/**
 * Copyright (c) Web Loupe. All rights reserved.
 * This file is part of software that is released under a proprietary license.
 * You must not copy, modify, distribute, make publicly available, or execute
 * its contents or parts thereof without express permission by the copyright
 * holder, unless otherwise permitted by law.
 */

namespace WeloConsorsFinanz\Subscriber\Backend;

use Enlight\Event\SubscriberInterface;
use Enlight_Event_EventArgs;
use Enlight_View_Default;
use Shopware\Components\DependencyInjection\Container as DIContainer;
use Shopware\Components\Model\ModelManager;
use Shopware\Models\Order\Order as OrderEntity;
use Shopware\Models\Order\Status;
use Shopware\Models\Shop\Shop;
use WeloConsorsFinanz\Components\Configuration;
use WeloConsorsFinanz\Components\Consors\Api\ConsorsSoapClientFault;
use WeloConsorsFinanz\Components\Consors\Services\ChangeService;
use WeloConsorsFinanz\Components\Consors\Services\OrderStornoService;
use WeloConsorsFinanz\Components\Logger\LoggerServiceInterface;
use WeloConsorsFinanz\Components\SettingsTable;
use Shopware\Models\Shop\Repository as ShopRepository;

/**
 * Class Order
 *
 * @author    Cyprien Nkeneng <cyprien.nkeneng@webloupe.de> - www.webloupe.de
 * @copyright Copyright (c) 2017-2020 WEB LOUPE
 * @package   WeloConsorsFinanz\Subscriber\Backend
 * @link      https://www.webloupe.de
 * @version   1
 */
class Order implements SubscriberInterface
{
    /**
     * @var Configuration
     */
    private $configuration;

    /**
     * @var OrderStornoService
     */
    private $orderStornoService;
    /**
     * @var DIContainer
     */
    private $container;
    /**
     * @var LoggerServiceInterface
     */
    private $loggerService;
    /**
     * @var ChangeService
     */
    private $changeService;
    /**
     * @var ModelManager
     */
    private $modelManager;

    /**
     * Order constructor.
     * @param DIContainer            $container
     * @param LoggerServiceInterface $loggerService
     * @param ModelManager           $modelManager
     */
    public function __construct(
        DIContainer $container,
        LoggerServiceInterface $loggerService,
        ModelManager $modelManager
    ) {
        $this->container = $container;

//        $this->registerShopResource();
//        $this->configuration = $container->get('welo_consors_finanz.components.configuration');
//        $this->orderStornoService = $container->get('welo_consors_finanz.components.consors.services.order_storno_service');
//        $this->changeService = $container->get('welo_consors_finanz.components.consors.services.change_service');
        $this->loggerService = $loggerService;
        $this->modelManager = $modelManager;
    }

    public static function getSubscribedEvents()
    {
        return [
            'Enlight_Controller_Action_PostDispatchSecure_Backend_Order' => ['onBackendOrderPostDispatch']
        ];
    }

    /**
     * Called when the BackendOrderPostDispatch Event is triggered
     * @param Enlight_Event_EventArgs $args
     */
    public function onBackendOrderPostDispatch(Enlight_Event_EventArgs $args)
    {
        $controller = $args->getSubject();

        /** @var Enlight_View_Default $view */
        $view = $controller->View();
        $request = $controller->Request();

//        error_log(__METHOD__ . '::' . __LINE__ . '::> ' . print_r($request->getActionName(), 1));
        switch ($request->getActionName()) {
//            case 'save':
//                $data = $view->getAssign('data');
//                if (0 !== (int) $data['id'] && 0 !== (int) $data['shopId']) {
//                    $this->updateOrderState($data);
//                }
//                break;
//            case 'deletePosition':
//            case 'savePosition':
//                $orderId = $request->getParam('orderId');
//
//                /** @var OrderEntity $order */
//                $order = $this->modelManager->getRepository(OrderEntity::class)->find($orderId);
//
//                if (!$order) {
//                    $this->loggerService->error(sprintf('Order with id "%s" not found.', $orderId));
//
//                    throw new \RuntimeException(
//                        sprintf(
//                            'Order with id "%s" not found.',
//                            $orderId
//                        )
//                    );
//                }
//                $this->updateOrder($order);
//                break;
            case 'load':
                $view->extendsTemplate('backend/order/consors_finanz/controller/detail.js');
                $view->extendsTemplate('backend/order/consors_finanz/view/detail/consors_finanz.js');
                break;
        }
    }

    /**
     * @param OrderEntity|null $order
     * @throws ConsorsSoapClientFault
     */
    private function updateOrder(OrderEntity $order = null)
    {
        if (null == $order) {
            $this->loggerService->error('Order not found');
            return;
        }
        $wConsorsFinanz = $this->configuration->getPluginData();
        $wConsorsFinanzPaymentIds = [$wConsorsFinanz['invoicePaymentMethodId'], $wConsorsFinanz['debitPaymentMethodId']];

        $paymentId = $order->getPayment()->getId();

        if (0 === (int)$paymentId || !in_array($paymentId, $wConsorsFinanzPaymentIds)) {
            $this->loggerService->error('not consors payment');
            return;
        }

        $this->loggerService->error('Order number = ' . $order->getNumber());

        if ('' == $order->getNumber()) {
            $this->loggerService->error('Order number empty');
            return;
        }

        $orderChangeResponse = $this->changeService->change($order->getNumber(), SettingsTable::CHANGE_TYPE_BUCHUNG_BONICHECK);
        if (0 != $orderChangeResponse->getFehlergrund()) {
            $this->loggerService->error($orderChangeResponse->getFehlertext() . ' (' . $order->getNumber() . ')');
        } else {
            $this->loggerService->error(sprintf('Order %s successfully changed', $order->getNumber()));
        }
    }

    /**
     * @param array $data
     * @throws ConsorsSoapClientFault
     * @throws \Exception
     */
    private function updateOrderState(array $data)
    {
        $wConsorsFinanz = $this->configuration->getPluginData();
        $wConsorsFinanzPaymentIds = [$wConsorsFinanz['invoicePaymentMethodId'], $wConsorsFinanz['debitPaymentMethodId']];

        if (!in_array($data['paymentId'], $wConsorsFinanzPaymentIds)) {
            return;
        }

        if ('' == $data['number']) {
            $this->loggerService->error('Order number empty');
            return;
        }

        switch ($data['status']) {
            case Status::ORDER_STATE_COMPLETELY_DELIVERED:  {
                // Now begin the payment
                $orderChangeResponse = $this->changeService->change($data['number'], SettingsTable::CHANGE_TYPE_BUCHUNG_BONICHECK);
                if (0 != $orderChangeResponse->getFehlergrund()) {
                    $this->loggerService->error($orderChangeResponse->getFehlertext() . ' (' . $data['number'] . ')');
                } else {
                    $this->loggerService->error(sprintf('Order %s successfully changed', $data['number']));
                }
            }
            break;
            case Status::ORDER_STATE_CANCELLED_REJECTED: {
                // If the oder has been cancelled, we cancel the order2 as well
                if ($data['paymentId'] == $wConsorsFinanz['invoicePaymentMethodId']) {
                    $requestType = SettingsTable::REQUEST_TYPE_INVOICE;
                } else {
                    $requestType = SettingsTable::REQUEST_TYPE_DEBIT;
                }

                $orderStornoResponse = $this->orderStornoService->cancel($data['number'], $requestType);
                if (0 != $orderStornoResponse->getFehlergrund()) {
                    $this->loggerService->error($orderStornoResponse->getFehlertext() . ' (' . $data['number'] . ')');
                } else {
                    $this->loggerService->error(sprintf('Order %s successfully cancelled', $data['number']));
                }
            }
            break;
        }
    }
}