<?php
/*
 * Copyright (c) Web Loupe. All rights reserved.
 * This file is part of software that is released under a proprietary license.
 * You must not copy, modify, distribute, make publicly available, or execute
 * its contents or parts thereof without express permission by the copyright
 * holder, unless otherwise permitted by law.
 */

namespace WeloConsorsFinanz\Subscriber;

use Enlight\Event\SubscriberInterface;
use Shopware\Bundle\AttributeBundle\Service\DataLoaderInterface;
use Shopware\Bundle\AttributeBundle\Service\DataPersisterInterface;
use Shopware\Components\DependencyInjection\Container as DIContainer;
use Shopware\Components\Model\ModelManager;
use Shopware\Models\Order\Document\Document;
use Shopware\Models\Shop\Repository as ShopRepository;
use Shopware\Models\Shop\Shop;
use WeloConsorsFinanz\Components\Configuration;
use WeloConsorsFinanz\Components\Consors\Services\ChangeService;
use WeloConsorsFinanz\Components\Consors\Services\OrderStornoService;
use WeloConsorsFinanz\Components\Logger\LoggerServiceInterface;
use WeloConsorsFinanz\Components\SettingsTable;

/**
 * Class Cron
 *
 * @author    Cyprien Nkeneng <cyprien.nkeneng@webloupe.de> - www.webloupe.de
 * @copyright Copyright (c) 2017-2020 WEB LOUPE
 * @package   WeloConsorsFinanz\Subscriber
 * @link      https://www.webloupe.de
 * @version   1
 */
class Cron implements SubscriberInterface
{
    /**
     * @var Configuration
     */
    private $configuration;
    /**
     * @var ModelManager
     */
    private $modelManager;
    /**
     * @var DIContainer
     */
    private $container;
    /**
     * @var LoggerServiceInterface
     */
    private $loggerService;

    /**
     * @var OrderStornoService
     */
    private $orderStornoService;

    /**
     * @var ChangeService
     */
    private $changeService;

    /**
     * @var DataLoaderInterface
     */
    private $dataLoader;
    /**
     * @var DataPersisterInterface
     */
    private $dataPersister;

    /**
     * Cron constructor.
     * @param Configuration $configuration
     * @param ModelManager  $modelManager
     * @param DIContainer   $container
     */
    public function __construct(
        Configuration $configuration,
        ModelManager $modelManager,
        DIContainer $container
    ) {
        $this->configuration = $configuration;
        $this->modelManager = $modelManager;
        $this->container = $container;

        $this->registerShopResource();
        $this->loggerService = $container->get('welo_consors_finanz.components.logger.logger_service');
        $this->orderStornoService = $container->get('welo_consors_finanz.components.consors.services.order_storno_service');
        $this->changeService = $container->get('welo_consors_finanz.components.consors.services.change_service');
        $this->dataLoader = $container->get('shopware_attribute.data_loader');
        $this->dataPersister = $container->get('shopware_attribute.data_persister');
    }

    /**
     * {@inheritdoc}
     */
    public static function getSubscribedEvents()
    {
        return [
            'Shopware_CronJob_WeloConsorsFinanz_OrderChangeCronJob' => 'onOrderChangeCronJob',
        ];
    }

    /**
     * @return string|void
     */
    public function onOrderChangeCronJob()
    {
        $output = PHP_EOL;
        $counter = 0;
        $occurredErrors = PHP_EOL;

        $wConsorsFinanz = $this->configuration->getPluginData();
        $wConsorsFinanzPaymentIds = [$wConsorsFinanz['invoicePaymentMethodId'], $wConsorsFinanz['debitPaymentMethodId']];

        $documents = $this->getOderDocuments();
        foreach ($documents as $document) {
            $documentType = $document['type'];
            $order = $document['order'];
            $payment = $document['order']['payment'];
            if (!in_array($payment['id'], $wConsorsFinanzPaymentIds)) {
                continue;
            }

            $orderAttribute = $this->dataLoader->load('s_order_attributes', $order['id']);
            $counter = 0;
            switch ($documentType['name']) {
                case 'Rechnung': {
                    if ($document['amount'] == $order['invoiceAmount']) {
//                    $this->dumb($document);

                        // if not booked
                        if (false == (bool)$orderAttribute['consors_finanz_is_order_booked']) {
                            // Buchung
                            $orderChangeResponse = $this->changeService->change($document, SettingsTable::CHANGE_TYPE_BUCHUNG_BONICHECK);
                            if (0 != $orderChangeResponse->getFehlergrund()) {
                                $this->loggerService->error($orderChangeResponse->getFehlertext() . ' (' . $order['number'] . ')');
                            } else {
                                $this->loggerService->error(sprintf('Order %s successfully changed', $order['number']));
                                // here book
                                $this->dataPersister->persist(['consors_finanz_is_order_booked' => true], 's_order_attributes', $order['id']);
                                ++$counter;
                            }
                        }
                    }
                    break;
                }
                case 'Stornorechnung': {
                    // if not yet cancelled
                    if (false == (bool)$orderAttribute['consors_finanz_is_order_cancelled']) {
                        // If the oder has been cancelled, we cancel the order2 as well
                        if ($payment['id'] == $wConsorsFinanz['invoicePaymentMethodId']) {
                            $requestType = SettingsTable::REQUEST_TYPE_INVOICE;
                        } else {
                            $requestType = SettingsTable::REQUEST_TYPE_DEBIT;
                        }

                        $orderStornoResponse = $this->orderStornoService->cancel($order['number'], $requestType);
                        if (0 != $orderStornoResponse->getFehlergrund()) {
                            $this->loggerService->error($orderStornoResponse->getFehlertext() . ' (' . $order['number'] . ')');
                        } else {
                            $this->loggerService->error(sprintf('Order %s successfully cancelled', $order['number']));
                            $this->dataPersister->persist(['consors_finanz_is_order_cancelled' => true], 's_order_attributes', $order['id']);
                            ++$counter;
                        }
                    }
                    break;
                }
                case 'Gutschrift': {
                    $creditNumbers = $orderAttribute['consors_finanz_order_credit_numbers'];
                    $arrayCreditNumbers = explode(',', $creditNumbers);
                    if (!in_array($document['documentId'], $arrayCreditNumbers)) {
                        // Gutschrift
                        $orderChangeResponse = $this->changeService->change($document,
                            SettingsTable::CHANGE_TYPE_BUCHUNG_BONICHECK, true);
                        if (0 != $orderChangeResponse->getFehlergrund()) {
                            $this->loggerService->error($orderChangeResponse->getFehlertext() . ' (' . $order['number'] . ')');
                        } else {
                            $this->loggerService->error(sprintf('Order %s successfully changed', $order['number']));

                            $newNumbers = '' == $creditNumbers ? $document['documentId'] : $creditNumbers . ',' . $document['documentId'];
                            $this->dataPersister->persist(['consors_finanz_order_credit_numbers' => $newNumbers],
                                's_order_attributes', $order['id']);
                            ++$counter;
                        }
                    }
                    break;
                }
            }
        }

        $output .= sprintf(
            '%sWeloConsorsFinanz: %s order-change processed. %s %s',
            $this->getNewLine(),
            $counter,
            $this->getNewLine(),
            $occurredErrors
        );

        echo $output;
        return $output;
    }

    /**
     * @return array
     */
    private function getOderDocuments()
    {
        $documents = $this->modelManager->getRepository(Document::class)->createQueryBuilder('document')
            ->select('document', 'documentType', 'swOrder', 'details', 'customer', 'billing', 'shipping', 'payment')
            ->leftJoin('document.type', 'documentType')
            ->leftJoin('document.order', 'swOrder')
            ->leftJoin('swOrder.details', 'details')
            ->leftJoin('swOrder.customer', 'customer')
            ->leftJoin('swOrder.billing', 'billing')
            ->leftJoin('swOrder.shipping', 'shipping')
            ->leftJoin('swOrder.payment', 'payment')
            ->getQuery()
            ->getArrayResult()
        ;

        if ($documents) {
            foreach ($documents as &$document) {
                $document['amount'] = $document['amount'] * 100;
                $document['order']['invoiceAmount'] = $document['order']['invoiceAmount'] * 100;
                $document['order']['invoiceAmountNet'] = $document['order']['invoiceAmountNet'] * 100;
                $document['order']['invoiceShipping'] = $document['order']['invoiceShipping'] * 100;
                $document['order']['invoiceShippingNet'] = $document['order']['invoiceShippingNet'] * 100;

                foreach ($document['order']['details'] as &$detail) {
                    $detail['price'] = $detail['price'] * 100;
                }
            }
        }

        return $documents;
    }


    private function registerShopResource()
    {
        /** @var ShopRepository $shopRepository */
        $shopRepository = $this->container->get('models')->getRepository(Shop::class);

        $shop = $shopRepository->getActiveById(1);
        if ($shop === null) {
            $shop = $shopRepository->getActiveDefault();
        }

        if ($this->container->has('shopware.components.shop_registration_service')) {
            $this->container->get('shopware.components.shop_registration_service')->registerResources($shop);
        } else {
            $shop->registerResources();
        }
    }

    private function getNewLine()
    {
        return PHP_SAPI == 'cli' ? "\n" : '<br>';
    }
}