<?php

namespace MilleConsorsFinanz\Components\Client;

use MilleConsorsFinanz\Components\Services\LogServiceInterface;
use MilleConsorsFinanz\Components\Settings\SettingsProvider;
use MilleConsorsFinanz\Models\Financing\Financing;
use Shopware\Components\HttpClient\GuzzleFactory;
use Shopware\Components\HttpClient\GuzzleHttpClient;
use Shopware\Components\HttpClient\RequestException;

class ApiClient implements ApiClientInterface
{
    /**
     * @var GuzzleHttpClient
     */
    private $client;

    /**
     * @var SettingsProvider
     */
    private $settingsProvider;

    /**
     * @var LogServiceInterface
     */
    private $logger;

    /**
     * @param SettingsProvider $settingsProvider
     * @param GuzzleFactory $guzzleFactory
     * @param LogServiceInterface $logger
     */
    public function __construct(
        SettingsProvider $settingsProvider,
        GuzzleFactory $guzzleFactory,
        LogServiceInterface $logger
    ) {
        $this->settingsProvider = $settingsProvider;
        $this->logger = $logger;

        $this->client = new GuzzleHttpClient($guzzleFactory);
    }

    /**
     * @param Financing $financing
     *
     * @throws RequestException
     */
    public function updateFinancingOrderNumber(Financing $financing)
    {
        $financingContext = $financing->getFinancingContext();

        $url = $this->fillUrlTemplate(
            ApiUrl::HOST[$financingContext->getCountry()] . ApiUrl::UPDATE_ORDER_NUMBER,
            ['clientid' => $this->settingsProvider->getClientId($financingContext)]
        );

        $headers = $this->getJsonHeaders($this->authenticateVendor($financing));

        $content = json_encode([
            'subscriptionIdentifierExternal' => $financing->getOrderNumber(),
            'transactionId' => $financing->getTransactionId()
        ]);

        $this->logger->info(
            'API request - update financing order number (' . $financing->getTransactionId() . ')',
            [
                'url' => $url,
                'content' => $content
            ],
            $financingContext
        );

        $response = $this->client->put($url, $headers, $content);

        $this->logger->info('API response - update financing order number (' . $financing->getTransactionId() . ')',
            [
                'status' => $response->getStatusCode(),
                'body' => $response->getBody()
            ],
            $financingContext
        );

        if (!in_array($response->getStatusCode(), [200, 201, 202])) {
            throw new RequestException();
        }
    }

    /**
     * @param Financing $financing
     *
     * @throws RequestException
     */
    public function cancelFinancing(Financing $financing)
    {
        $financingContext = $financing->getFinancingContext();

        $url = $this->fillUrlTemplate(
            ApiUrl::HOST[$financingContext->getCountry()] . ApiUrl::CANCEL_FINANCING,
            [
                'clientid' => $this->settingsProvider->getClientId($financingContext),
                'transactionId' => $financing->getTransactionId()
            ]
        );

        $headers = $this->getJsonHeaders($this->authenticateVendor($financing));

        $content = json_encode([
            'subscriptionIdentifierExternal' => $financing->getOrderNumber(),
            'transactionId' => $financing->getTransactionId()
        ]);

        $this->logger->info(
            'API request - cancel financing (' . $financing->getTransactionId() . ')',
            [
                'url' => $url,
                'content' => $content
            ],
            $financingContext
        );

        $response = $this->client->delete($url, $headers);

        $this->logger->info('API response - cancel financing (' . $financing->getTransactionId() . ')',
            [
                'status' => $response->getStatusCode(),
                'body' => $response->getBody()
            ],
            $financingContext
        );

        if (!in_array($response->getStatusCode(), [200, 201, 202])) {
            throw new RequestException();
        }
    }

    /**
     * @param Financing $financing
     *
     * @throws RequestException
     */
    public function approveFinancing(Financing $financing)
    {
        $financingContext = $financing->getFinancingContext();

        $url = $this->fillUrlTemplate(
            ApiUrl::HOST[$financingContext->getCountry()] . ApiUrl::APPROVE_FINANCING,
            [
                'clientid' => $this->settingsProvider->getClientId($financingContext),
                'transactionId' => $financing->getTransactionId()
            ]
        );

        $headers = $this->getJsonHeaders($this->authenticateVendor($financing));

        $this->logger->info(
            'API request -approve financing (' . $financing->getTransactionId() . ')',
            ['url' => $url],
            $financingContext
        );

        $response = $this->client->put($url, $headers);

        $this->logger->info('API response - approve financing (' . $financing->getTransactionId() . ')',
            [
                'status' => $response->getStatusCode(),
                'body' => $response->getBody()
            ],
            $financingContext
        );

        if (!in_array($response->getStatusCode(), [200, 201, 202])) {
            throw new RequestException();
        }
    }

    /**
     * @param Financing $financing
     *
     * @return string
     * @throws RequestException
     */
    private function authenticateVendor(Financing $financing): string
    {
        $financingContext = $financing->getFinancingContext();

        $url = $this->fillUrlTemplate(
            ApiUrl::HOST[$financingContext->getCountry()] . ApiUrl::AUTHENTICATE_VENDOR,
            ['clientid' => $this->settingsProvider->getClientId($financingContext)]
        );

        $headers = [
            'Content-Type' => 'application/x-www-form-urlencoded'
        ];

        $content = [
            'username' => $this->settingsProvider->getUsername($financingContext),
            'password' => $this->settingsProvider->getVendorPassword($financingContext)
        ];

        $this->logger->info(
            'API request - authenticate vendor (' . $financing->getTransactionId() . ')',
            [
                'url' => $url,
                'content' => $content
            ],
            $financingContext
        );

        $response = $this->client->post($url, $headers, $content);

        $this->logger->info('API response - authenticate vendor (' . $financing->getTransactionId() . ')',
            [
                'status' => $response->getStatusCode(),
                'body' => $response->getBody()
            ],
            $financingContext
        );

        if (!in_array($response->getStatusCode(), [200, 201, 202])) {
            throw new RequestException();
        }

        $responseBody = json_decode($response->getBody(), true);

        return $responseBody['token'];
    }

    /**
     * @param string $token
     *
     * @return array
     */
    private function getJsonHeaders(string $token): array
    {
        return [
            'Content-Type' => 'application/json',
            'Accept' => 'application/json',
            'Authorization' => $token
        ];
    }

    /**
     * @param string $template
     * @param array $values
     *
     * @return string
     */
    private function fillUrlTemplate(string $template, array $values): string
    {
        $url = $template;

        foreach ($values as $variable => $value) {
            $url = str_replace('{' . $variable . '}', $value, $url);
        }

        return $url;
    }
}
