<?php

namespace MilleConsorsFinanz\Components\Services;

use DateTime;
use MilleConsorsFinanz\Components\PaymentStatus;
use MilleConsorsFinanz\Models\Financing\Financing;
use MilleConsorsFinanz\Models\Financing\Repository as FinancingRepository;
use Shopware\Components\Model\ModelManager;
use Shopware\Models\Order\Order;
use Shopware\Models\Order\Repository as OrderRepository;
use Shopware\Models\Order\Status;

class AbortedFinancingsCleanupService
{
    /**
     * @var int threshold in hours
     */
    private $timeConstraint = 12;

    /**
     * @var FinancingRepository
     */
    private $financingRepository;

    /**
     * @var OrderRepository
     */
    private $orderRepository;

    /**
     * @var ModelManager
     */
    private $modelManager;

    /**
     * @var StatusService
     */
    private $statusService;

    /**
     * @param ModelManager $modelManager
     * @param StatusService $statusService
     */
    public function __construct(
        ModelManager $modelManager,
        StatusService $statusService
    ) {
        $this->modelManager = $modelManager;
        $this->statusService = $statusService;

        $this->financingRepository = $modelManager->getRepository(Financing::class);
        $this->orderRepository = $modelManager->getRepository(Order::class);
    }

    /**
     * Cancels financing requests, which were aborted on the Consors Finanz page, i.e. the user closed the page in his
     * browser. In this case Consors Finanz cancels the request after a timeout but sends no status update notification.
     */
    public function cancelAbortedFinancingRequests()
    {
        $analysingStatus = $this->statusService->findStatusByName(PaymentStatus::ANALYSING);
        $cancelledStatus = $this->statusService->findStatusByName(PaymentStatus::CANCELLED);
        $timeConstraint = new DateTime("- {$this->timeConstraint} hours");
        $financings = $this->financingRepository->getStatusAndMaxTimeQuery($analysingStatus->getId(), $timeConstraint)
            ->getResult();

        if (!$financings) {
            return;
        }

        /** @var Financing $financing */
        foreach ($financings as $financing) {
            $this->updateFinancingStatusAndDate($financing, $cancelledStatus);

            if ($financing->hasOrder()) {
                $this->updateOrderStatus($financing->getOrderNumber(), $cancelledStatus);
            }
        }
    }

    /**
     * @param Financing $financing
     * @param Status $status
     */
    private function updateFinancingStatusAndDate(Financing $financing, Status $status)
    {
        $financing->setStatus($status->getId());
        $financing->setUpdatedTimeToNow();

        $this->modelManager->persist($status);
        $this->modelManager->flush();
    }

    /**
     * @param int $orderNumber
     * @param Status $cancelledStatus
     */
    private function updateOrderStatus(int $orderNumber, Status $cancelledStatus)
    {
        /** @var Order $order */
        $order = $this->orderRepository->findOneBy(['number' => $orderNumber]);

        if (!$order) {
            return;
        }

        $orderModule = Shopware()->Modules()->Order();
        $orderModule->setPaymentStatus($order->getId(), $cancelledStatus->getId(), true);
    }
}
