<?php

namespace MilleConsorsFinanz\Components\Services;

use Doctrine\DBAL\Connection;
use MilleConsorsFinanz\Components\Settings\SettingsProvider;
use Shopware\Components\Model\ModelManager;
use Shopware\Models\Article\Article;
use Shopware\Models\Article\Repository as ArticleRepository;
use Shopware\Models\Category\Category;

class CampaignService
{
    /**
     * @var SettingsProvider
     */
    private $settingsProvider;

    /**
     * @var Connection
     */
    private $connection;

    /**
     * @var ArticleRepository
     */
    private $articleRepository;

    /**
     * @param ModelManager $modelManager
     * @param SettingsProvider $settingsProvider
     * @param Connection $connection
     */
    public function __construct(
        ModelManager $modelManager,
        SettingsProvider $settingsProvider,
        Connection $connection
    ) {
        $this->settingsProvider = $settingsProvider;
        $this->connection = $connection;

        $this->articleRepository = $modelManager->getRepository(Article::class);
    }

    /**
     * @param int $articleId
     *
     * @return bool
     */
    public function isCampaignArticle(int $articleId): bool
    {
        $campaignCategoryId = $this->settingsProvider->getCampaignCategoryId();

        if (!$campaignCategoryId) {
            return true;
        }

        /** @var Article $article */
        $article = $this->articleRepository->find($articleId);
        $articleCategories = $article->getAllCategories();

        /** @var Category $category */
        foreach ($articleCategories as $category) {
            if ($category->getId() == $campaignCategoryId) {
                return true;
            }
        }

        return false;
    }

    /**
     * @param array $basketContent
     *
     * @return bool
     */
    public function containsBasketOnlyCampaignArticles(array $basketContent): bool
    {
        return $this->containsBasketCampaignArticles($basketContent)
            && !$this->containsBasketNonCampaignArticles($basketContent);
    }

    /**
     * @param array $basketContent
     *
     * @return bool
     */
    public function containsBasketMixedArticles(array $basketContent): bool
    {
        return $this->containsBasketCampaignArticles($basketContent)
            && $this->containsBasketNonCampaignArticles($basketContent);
    }

    /**
     * @return array associative array with campaign interest rates and durations, e.g.
     *      [
     *          'interestRates' => [0, 0, 9.5],
     *          'durations' => [6, 12, 18]
     *      ]
     */
    public function getCampaignInterestRatesAndDurations(): array
    {
        $campaignInterestRate = $this->settingsProvider->getCampaignInterestRate();
        $campaignId = $this->settingsProvider->getCampaignId();
        $campaignDuration = $this->settingsProvider->getCampaignDuration();
        $durations = $this->settingsProvider->getDurations();
        $interestRates = $this->settingsProvider->getInterestRates();

        if (!$campaignId || !$campaignDuration) {
            return [
                'interestRates' => $interestRates,
                'durations' => $durations
            ];
        }

        if ($campaignId == 3) {
            return [
                'interestRates' => [$campaignInterestRate],
                'durations' => [$campaignDuration]
            ];
        }

        for ($i = 0; $i < count($durations); $i++) {
            switch ($campaignId) {
                case 1:
                    if ($campaignDuration == $durations[$i]) {
                        $interestRates[$i] = $campaignInterestRate;
                    }
                    break;

                case 2:
                    if ($campaignDuration >= $durations[$i]) {
                        $interestRates[$i] = $campaignInterestRate;
                    }
                    break;
            }
        }

        return [
            'interestRates' => $interestRates,
            'durations' => $durations
        ];
    }

    /**
     * @param array $basketContent
     *
     * @return bool
     */
    private function containsBasketCampaignArticles(array $basketContent): bool
    {
        foreach ($basketContent as $article) {
            if ($article['modus'] == '0' && $this->isCampaignArticle($article['articleID'])) {
                return true;
            }
        }

        return false;
    }

    /**
     * @param array $basketContent
     *
     * @return bool
     */
    private function containsBasketNonCampaignArticles(array $basketContent): bool
    {
        foreach ($basketContent as $article) {
            if ($article['modus'] == '0' && !$this->isCampaignArticle($article['articleID'])) {
                return true;
            }
        }

        return false;
    }
}
