<?php

namespace MilleConsorsFinanz\Components\Services;

use MilleConsorsFinanz\Components\FinancingExampleMode;
use MilleConsorsFinanz\Components\Settings\SettingsProvider;
use MilleConsorsFinanz\Struct\FinancingOption;

class InstallmentsCalculator
{
    /**
     * @var SettingsProvider
     */
    private $settingsProvider;

    /**
     * @var CampaignService
     */
    private $campaignService;

    /**
     * @param SettingsProvider $settingsProvider
     * @param CampaignService $campaignService
     */
    public function __construct(
        SettingsProvider $settingsProvider,
        CampaignService $campaignService)
    {
        $this->settingsProvider = $settingsProvider;
        $this->campaignService = $campaignService;
    }

    /**
     * @param int $articleId
     * @param float $price
     *
     * @return FinancingOption|null
     */
    public function getFinancingExample(int $articleId, float $price)
    {
        $financingExampleMode = $this->settingsProvider->getFinancingExampleMode();
        $financingOption = null;
        $financingOptions = $this->getFinancingOptions($articleId, $price);

        if (FinancingExampleMode::LOWEST_ZERO_PERCENT_INSTALLMENT == $financingExampleMode) {
            $financingOption = $this->getFinancingOptionWithLowestZeroPercentInstallment($financingOptions);
        }

        if (!$financingOption) {
            $financingOption = $this->getFinancingOptionWithLowestInstallment($financingOptions);
        }

        return $financingOption;
    }

    /**
     * @param FinancingOption[] $financingOptions
     *
     * @return FinancingOption|null
     */
    public function getFinancingOptionWithLowestInstallment(array $financingOptions)
    {
        $lowestInstallment = INF;
        $option = null;

        foreach ($financingOptions as $financingOption) {
            if ($financingOption->getInstallment() < $lowestInstallment) {
                $option = $financingOption;
                $lowestInstallment = $financingOption->getInstallment();
            }
        }

        return $option;
    }

    /**
     * @param FinancingOption[] $financingOptions
     *
     * @return FinancingOption|null
     */
    public function getFinancingOptionWithLowestZeroPercentInstallment(array $financingOptions)
    {
        $lowestInstallment = INF;
        $option = null;

        foreach ($financingOptions as $financingOption) {
            if ($financingOption->getInstallment() < $lowestInstallment &&
                    $financingOption->getInterestRate() == 0) {
                $option = $financingOption;
                $lowestInstallment = $financingOption->getInstallment();
            }
        }

        return $option;
    }

    /**
     * @param int $articleId
     * @param float $price
     *
     * @return FinancingOption[]
     */
    public function getFinancingOptions(int $articleId, float $price): array
    {
        $financingOptions = [];
        $isCampaignArticle = $this->campaignService->isCampaignArticle($articleId);

        if ($isCampaignArticle) {
            $campaignInterestRatesAndDurations = $this->campaignService->getCampaignInterestRatesAndDurations();
            $durations = $campaignInterestRatesAndDurations['durations'];
            $interestRates = $campaignInterestRatesAndDurations['interestRates'];
        } else {
            $durations = $this->settingsProvider->getDurations();
            $interestRates = $this->settingsProvider->getInterestRates();
        }

        for ($i = 0; $i < count($durations); $i++) {
            $financingOption = $this->getFinancingOption($price, $durations[$i], $interestRates[$i]);

            if ($financingOption->getInstallment() >= $this->settingsProvider->getMinInstallment()) {
                $financingOptions[] = $financingOption;
            }
        }

        return $financingOptions;
    }

    /**
     * @param float $price
     * @param int $duration
     * @param float $interestRate
     *
     * @return FinancingOption|null
     */
    public function getFinancingOption(float $price, int $duration, float $interestRate)
    {
        $installment = $this->getInstallment($price, $duration, $interestRate);
        $totalAmount = $this->getFinancingTotalAmount($installment, $duration);
        $borrowingRate = $this->getBorrowingRate($interestRate);

        $financingOption = new FinancingOption();
        $financingOption->setInstallment($installment);
        $financingOption->setTotalAmount($totalAmount);
        $financingOption->setBorrowingRate($borrowingRate);
        $financingOption->setDuration($duration);
        $financingOption->setInterestRate($interestRate);

        return $financingOption;
    }

    /**
     * @param float $interestRate
     *
     * @return float
     */
    public function getBorrowingRate(float $interestRate): float
    {
        // Sollzinssatz = ((1+Zinssatz_Effektiv/100)^(1/12)-1)*12*100
        return $this->truncateFloat((pow(1 + $interestRate / 100, 1 / 12) - 1) * 12 * 100);
    }

    /**
     * @param float $price
     * @param float $interestRate
     * @param int $duration
     *
     * @return float
     */
    public function getInstallment(float $price, int $duration, float $interestRate): float
    {
        if ($interestRate == 0) {
            $installment = $price / $duration;
        } else {
            // Rate = Kapital*((Zinssatz_Effektiv/100+1)^(1/12))^Monate / ((((Zinssatz_Effektiv/100+1)^(1/12))^Monate)-1) * (((Zinssatz_Effektiv/100+1)^(1/12))-1)
            $interestRateFactor = $interestRate / 100 + 1;              // (Zinssatz_Effektiv/100+1)
            $interestFactor = pow($interestRateFactor, 1 / 12);    // (Zinssatz_Effektiv/100+1)^(1/12)
            $monthlyFactor = pow($interestFactor, $duration);           // (Zinssatz_Effektiv/100+1)^(1/12))^Monate
            $installment = $price * $monthlyFactor / ($monthlyFactor - 1) * ($interestFactor - 1);
        }

        return $this->truncateFloat($installment);
    }

    /**
     * @param float $installment
     * @param int $duration
     *
     * @return float
     */
    public function getFinancingTotalAmount(float $installment, int $duration): float
    {
        return $this->truncateFloat($installment * $duration);
    }

    /**
     * Truncates a float to two decimal places, e.g. 9.999 -> 9.99
     *
     * @param float $number
     *
     * @return float
     */
    private function truncateFloat(float $number): float
    {
        return floor($number * 100) / 100;
    }
}
