<?php

namespace MilleConsorsFinanz\Components\Settings;

use MilleConsorsFinanz\Components\FinancingExampleMode;
use MilleConsorsFinanz\Models\Settings\Repository as SettingsRepository;
use MilleConsorsFinanz\Models\Settings\Settings;
use MilleConsorsFinanz\Struct\FinancingContextInterface;
use Shopware\Components\Model\ModelManager;
use Shopware\Models\Customer\Customer;
use Shopware\Models\Customer\Repository as CustomerRepository;

class SettingsProvider
{
    const MIN_ORDER_AMOUNT = 54.0;
    const MAX_ORDER_AMOUNT = 15000.0;
    const MIN_INSTALLMENT = 9.0;
    const CAMPAIGN_INTEREST_RATE = 0;

    const DEFAULT_COUNTRY_ISO = 'DE';
    const VALID_COUNTRY_ISOS = ['DE', 'AT'];

    /**
     * @var array map of country ISO to Settings object
     */
    private $settings;

    /**
     * @var SettingsRepository
     */
    private $settingsRepository;

    /**
     * @var CustomerRepository
     */
    private $customerRepository;

    /**
     * @param ModelManager $modelManager
     */
    public function __construct(
        ModelManager $modelManager)
    {
        $this->settingsRepository = $modelManager->getRepository(Settings::class);
        $this->customerRepository = $modelManager->getRepository(Customer::class);

        $this->settings = $this->getSettings();
    }

    /**
     * @param FinancingContextInterface $financingContext
     *
     * @return bool
     */
    public function isLiveMode(FinancingContextInterface $financingContext = null): bool
    {
        return $this->getSettingValue('liveMode', $financingContext) ?: false;
    }

    /**
     * @param FinancingContextInterface $financingContext
     *
     * @return bool
     */
    public function isLoggingEnabled(FinancingContextInterface $financingContext): bool
    {
        return $this->getSettingValue('logging', $financingContext) ?: false;
    }

    /**
     * @param FinancingContextInterface $financingContext
     *
     * @return string
     */
    public function getUsername(FinancingContextInterface $financingContext): string
    {
        $settingValue = $this->isLiveMode($financingContext) ?
            $this->getSettingValue('username', $financingContext) :
            TestData::USERNAME[$financingContext->getCountry()];

        return $settingValue ?: '';
    }

    /**
     * @param FinancingContextInterface $financingContext
     *
     * @return string
     */
    public function getClientId(FinancingContextInterface $financingContext): string
    {
        $settingValue = $this->isLiveMode($financingContext) ?
            $this->getSettingValue('clientId', $financingContext) :
            TestData::CLIENT_ID;

        return $settingValue ?: '';
    }

    /**
     * @param FinancingContextInterface $financingContext
     *
     * @return string
     */
    public function getVendorId(FinancingContextInterface $financingContext): string
    {
        if ($this->isLiveMode($financingContext)) {
            return $this->getSettingValue('vendorId', $financingContext);
        }

        return TestData::VENDOR_ID[$financingContext->getCountry()];
    }

    /**
     * @param FinancingContextInterface $financingContext
     *
     * @return string
     */
    public function getVendorPassword(FinancingContextInterface $financingContext): string
    {
        if ($this->isLiveMode($financingContext)) {
            return $this->getSettingValue('vendorPassword', $financingContext);
        }

        return TestData::VENDOR_PASSWORD[$financingContext->getCountry()];
    }

    /**
     * @param FinancingContextInterface $financingContext
     *
     * @return string
     */
    public function getHashKey(FinancingContextInterface $financingContext): string
    {
        $settingValue = $this->isLiveMode($financingContext) ?
            $this->getSettingValue('hashKey', $financingContext) :
            TestData::HASH_KEY;

        return $settingValue ?: '';
    }

    /**
     * @return string
     */
    public function getShopName(): string
    {
        return $this->getSettingValue('shopName') ?: '';
    }

    /**
     * @return string
     */
    public function getShopLogo(): string
    {
        return $this->getSettingValue('shopLogo') ?: '';
    }

    /**
     * @return float
     */
    public function getMinInstallment(): float
    {
        return self::MIN_INSTALLMENT;
    }

    /**
     * @return float
     */
    public function getMinOrderAmount(): float
    {
        $settingValue = $this->getSettingValue('minAmount');

        if (!$settingValue) {
            return self::MIN_ORDER_AMOUNT;
        }

        return max(self::MIN_ORDER_AMOUNT, (float) $settingValue);
    }

    /**
     * @return float
     */
    public function getMaxOrderAmount(): float
    {
        return self::MAX_ORDER_AMOUNT;
    }

    /**
     * @return bool
     */
    public function isCreateOrderBeforePayment(): bool
    {
        return $this->getSettingValue('createOrderBeforePayment') ?: false;
    }

    /**
     * @return int
     */
    public function getDefaultDuration(): int
    {
        return $this->getSettingValue('defaultDuration') ?: 0;
    }

    /**
     * @return array
     */
    public function getDurations(): array
    {
        $durations = $this->getSettingValue('durations');

        if (!$durations) {
            return [];
        }

        $durationsAsArrayOfIntegers = array_map(function(string $duration) {
            return (int) trim($duration);
        }, explode(',', $durations));

        return $durationsAsArrayOfIntegers;
    }

    /**
     * @return array
     */
    public function getInterestRates(): array
    {
        $interestRates = $this->getSettingValue('interestRates');

        if (!$interestRates) {
            return [];
        }

        $interestRatesAsArrayOfFloats = array_map(function(string $interestRate) {
            return (float) trim($interestRate);
        }, explode(',', $interestRates));

        return $interestRatesAsArrayOfFloats;
    }

    /**
     * @return int
     */
    public function getCampaignCategoryId(): int
    {
        return $this->getSettingValue('campaignCategoryId') ?: 0;
    }

    /**
     * @return int
     */
    public function getCampaignId(): int
    {
        return $this->getSettingValue('campaignId') ?: 0;
    }

    /**
     * @return int
     */
    public function getCampaignDuration(): int
    {
        return $this->getSettingValue('campaignDuration') ?: 0;
    }

    public function getFinancingExampleMode(): string
    {
        return $this->getSettingValue('financingExampleMode') ?: FinancingExampleMode::LOWEST_INSTALLMENT;
    }

    /**
     * @return array
     */
    public function getValidCountryIsos(): array
    {
        return self::VALID_COUNTRY_ISOS;
    }

    /**
     * @return float
     */
    public function getCampaignInterestRate(): float
    {
        return self::CAMPAIGN_INTEREST_RATE;
    }

    /**
     * @return array
     */
    private function getSettings(): array
    {
        $countryIsoToSettingsMap = [];
        $settingsResults = $this->settingsRepository->findAll();

        /** @var Settings $settingsResult */
        foreach ($settingsResults as $settingsResult) {
            $countryIsoToSettingsMap[$settingsResult->getCountryIso()] = $settingsResult;
        }

        return $countryIsoToSettingsMap;
    }

    /**
     * @param string $settingName
     *
     * @param FinancingContextInterface|null $financingContext
     *
     * @return mixed
     */
    private function getSettingValue(string $settingName, FinancingContextInterface $financingContext = null)
    {
        $billingCountryIso = $financingContext ?
            $financingContext->getCountry() : $this->getCustomerBillingCountryIsoFromSession();
        $settings = $this->getSettingsByCountry($billingCountryIso);

        if (!$settings) {
            return null;
        }

        $method = method_exists($settings, 'get' . ucwords($settingName)) ?
            'get' . ucwords($settingName) :
            'is' . ucwords($settingName);

        return $settings->{$method}();
    }

    /**
     * @param string $countryIso
     *
     * @return Settings|null
     */
    private function getSettingsByCountry(string $countryIso)
    {
        if (!$this->settings) {
            return null;
        }

        if (!$countryIso || !in_array($countryIso, self::VALID_COUNTRY_ISOS)) {
            return $this->settings[self::DEFAULT_COUNTRY_ISO];
        }

        return $this->settings[$countryIso];
    }

    /**
     * @return string
     */
    private function getCustomerBillingCountryIsoFromSession(): string
    {
        if (strpos(Shopware()->Front()->Request()->getRequestUri(), '/backend') !== false) {
            return self::DEFAULT_COUNTRY_ISO;
        }

        $sUserId = Shopware()->Session()->offsetGet('sUserId');

        if (!$sUserId) {
            return self::DEFAULT_COUNTRY_ISO;
        }

        /** @var Customer $customer */
        $customer = $this->customerRepository->find($sUserId);

        return $customer->getDefaultBillingAddress()->getCountry()->getIso();
    }
}
