<?php

use Doctrine\ORM\EntityRepository;
use Doctrine\ORM\OptimisticLockException;
use Doctrine\ORM\ORMException;
use MilleConsorsFinanz\Components\Client\ApiClient;
use MilleConsorsFinanz\Components\PaymentStatus;
use MilleConsorsFinanz\Components\Services\FinancingService;
use MilleConsorsFinanz\Models\Financing\Financing;
use Shopware\Components\HttpClient\RequestException;
use Shopware\Components\Model\ModelManager;
use Shopware\Models\Order\Order;
use Shopware\Models\Order\Repository as OrderRepository;
use Shopware\Models\Order\Status;

class Shopware_Controllers_Backend_MilleConsorsFinanz extends Shopware_Controllers_Backend_Application
{
    protected $model = Financing::class;
    protected $alias = 'financing';

    /**
     * @var ModelManager
     */
    private $modelManager;

    /**
     * @var FinancingService
     */
    private $financingService;

    /**
     * @var ApiClient
     */
    private $apiClient;

    /**
     * @var Shopware_Components_Snippet_Manager
     */
    private $snippets;

    /**
     * @var EntityRepository
     */
    private $statusRepository;

    /**
     * @var OrderRepository
     */
    private $orderRepository;

    public function preDispatch()
    {
        $this->modelManager = $this->getModelManager();
        $this->financingService = $this->get('mille_consors_finanz.financing_service');
        $this->apiClient = $this->get('mille_consors_finanz.api_client');
        $this->snippets = $this->get('snippets');
        $this->statusRepository = $this->modelManager->getRepository(Status::class);
        $this->orderRepository = $this->modelManager->getRepository(Order::class);

        parent::preDispatch();
    }

    /**
     * @throws OptimisticLockException
     */
    public function cancelFinancingAction()
    {
        $financingId = $this->Request()->getParam('financingId');

        /** @var Financing $financing */
        $financing = $this->getRepository()->find($financingId);

        if (!$financing) {
            $this->View()->assign([
                'success' => false,
                'message' => 'Anfrage nicht gefunden.'
            ]);
            return;
        }

        if (!$this->canCancelFinancing($financing->getStatus())) {
            $this->View()->assign([
                'success' => false,
                'message' => 'Anfrage kann nicht storniert werden.'
            ]);
            return;
        }

        try {
            $this->apiClient->cancelFinancing($financing);
        } catch (RequestException $e) {
            $this->View()->assign([
                'success' => false,
                'message' => $e->getMessage()
            ]);
            return;
        }

        $this->updateFinancingAndOrderStatus($financing, PaymentStatus::CANCELLED);

        $this->View()->assign(['success' => true]);
    }

    /**
     * @throws OptimisticLockException
     */
    public function approveFinancingAction()
    {
        $financingId = $this->Request()->getParam('financingId');

        /** @var Financing $financing */
        $financing = $this->getRepository()->find($financingId);

        if (!$financing) {
            $this->View()->assign([
                'success' => false,
                'message' => 'Anfrage nicht gefunden.'
            ]);
            return;
        }

        if (!$this->canApproveFinancing($financing->getStatus())) {
            $this->View()->assign([
                'success' => false,
                'message' => 'Anfrage kann nicht freigegeben werden.'
            ]);
            return;
        }

        try {
            $this->apiClient->approveFinancing($financing);
        } catch (RequestException $e) {
            $this->View()->assign([
                'success' => false,
                'message' => $e->getMessage()
            ]);
            return;
        }

        $this->updateFinancingAndOrderStatus($financing, PaymentStatus::DELIVERED);

        $this->View()->assign(['success' => true]);
    }

    /**
     * @param Financing $financing
     * @param string $paymentStatusName
     *
     * @throws OptimisticLockException
     * @throws ORMException
     */
    private function updateFinancingAndOrderStatus(Financing $financing, string $paymentStatusName)
    {
        $statusId = $this->statusRepository->findOneBy(['name' => $paymentStatusName])->getId();

        $financing->setStatus($statusId);
        $financing->setUpdatedTimeToNow();

        $this->modelManager->persist($financing);
        $this->modelManager->flush();

        /** @var Order $order */
        $order = $this->orderRepository->findOneBy(['number' => $financing->getOrderNumber()]);

        Shopware()->Modules()->Order()->setPaymentStatus($order->getId(), $statusId, true);
    }


    /**
     * {@inheritDoc}
     */
    protected function getList($offset, $limit, $sort = [], $filter = [], array $wholeParams = [])
    {
        $financings = parent::getList($offset, $limit, $sort, $filter, $wholeParams);

        foreach ($financings['data'] as &$financing) {
            /** @var Status $status */
            $status = $this->statusRepository->find($financing['status'] ?: 0);

            $financing['canCancel'] = $this->canCancelFinancing($status->getId());
            $financing['canApprove'] = $this->canApproveFinancing($status->getId());
            $financing['status'] = $this->snippets->getNamespace(
                'backend/static/payment_status')->get($status->getName());
        }

        return $financings;
    }

    /**
     * @param int $financingStatusId
     *
     * @return bool
     */
    private function canCancelFinancing(int $financingStatusId): bool
    {
        return $this->statusInArray($financingStatusId, [PaymentStatus::WAITING_FOR_DOCUMENTS, PaymentStatus::ACCEPTED]);
    }

    /**
     * @param int $financingStatusId
     *
     * @return bool
     */
    private function canApproveFinancing(int $financingStatusId): bool
    {
        return $this->statusInArray($financingStatusId, [PaymentStatus::ACCEPTED]);
    }

    /**
     * @param int $statusId
     * @param array $statusNames
     *
     * @return bool
     */
    private function statusInArray(int $statusId, array $statusNames): bool
    {
        if (!$statusId) {
            return false;
        }

        /** @var Status $status */
        $status = $this->statusRepository->find($statusId);

        return in_array($status->getName(), $statusNames);
    }
}
