<?php

use Doctrine\ORM\OptimisticLockException;
use Doctrine\ORM\ORMException;
use MilleConsorsFinanz\Components\Client\ApiClientInterface;
use MilleConsorsFinanz\Components\ErrorCode;
use MilleConsorsFinanz\Components\PaymentStatus;
use MilleConsorsFinanz\Components\Services\CampaignService;
use MilleConsorsFinanz\Components\Services\FinancingService;
use MilleConsorsFinanz\Components\Services\LogServiceInterface;
use MilleConsorsFinanz\Components\Services\NotificationService;
use MilleConsorsFinanz\Components\Services\StatusService;
use MilleConsorsFinanz\Components\Settings\SettingsProvider;
use MilleConsorsFinanz\Components\Url;
use MilleConsorsFinanz\Models\Financing\Financing;
use MilleConsorsFinanz\Models\Financing\Repository as FinancingRepository;
use MilleConsorsFinanz\Struct\FinancingRequest;
use Shopware\Components\CSRFWhitelistAware;
use Shopware\Components\Model\ModelManager;
use Shopware\Components\OptinServiceInterface;
use Shopware\Components\Routing\Router;
use Shopware\Models\Country\Country;
use Shopware\Models\Country\Repository as CountryRepository;
use Shopware\Models\Order\Order;
use Shopware\Models\Order\Repository as OrderRepository;
use Symfony\Component\HttpFoundation\Cookie;

class Shopware_Controllers_Frontend_MilleConsorsFinanz
    extends Shopware_Controllers_Frontend_Payment
    implements CSRFWhitelistAware
{
    const OPTIN_SESSION_PARAM = 'MilleConsorsFinanzOptinSessionParam';

    /**
     * @var ModelManager
     */
    private $modelManager;

    /**
     * @var SettingsProvider
     */
    private $settingsProvider;

    /**
     * @var FinancingService
     */
    private $financingService;

    /**
     * @var StatusService
     */
    private $statusService;

    /**
     * @var CampaignService
     */
    private $campaignService;

    /**
     * @var NotificationService
     */
    private $notificationService;

    /**
     * @var ApiClientInterface
     */
    private $apiClient;

    /**
     * @var LogServiceInterface
     */
    private $logger;

    /**
     * @var Router
     */
    private $router;

    /**
     * @var CountryRepository
     */
    private $countryRepository;

    /**
     * @var FinancingRepository
     */
    private $financingRepository;

    /**
     * @var OrderRepository
     */
    private $orderRepository;

    /**
     * @var Shopware_Components_Config
     */
    private $shopwareConfig;

    /**
     * @var Enlight_Components_Session_Namespace
     */
    private $session;

    /**
     * @throws Exception
     */
    public function preDispatch()
    {
        $this->modelManager = $this->getModelManager();
        $this->settingsProvider = $this->get('mille_consors_finanz.settings_provider');
        $this->financingService = $this->get('mille_consors_finanz.financing_service');
        $this->statusService = $this->get('mille_consors_finanz.status_service');
        $this->campaignService = $this->get('mille_consors_finanz.campaign_service');
        $this->notificationService = $this->get('mille_consors_finanz.notification_service');
        $this->apiClient = $this->get('mille_consors_finanz.api_client');
        $this->logger = $this->get('mille_consors_finanz.log_service');
        $this->router = $this->get('router');
        $this->shopwareConfig = $this->get('config');
        $this->session = $this->get('session');
        $this->countryRepository = $this->modelManager->getRepository(Country::class);
        $this->financingRepository = $this->modelManager->getRepository(Financing::class);
        $this->orderRepository = $this->modelManager->getRepository(Order::class);
    }

    /**
     * @throws Exception
     */
    public function indexAction()
    {
        if ($this->getPaymentShortName() != 'MilleConsorsFinanz') {
            return $this->redirect(['controller' => 'checkout']);
        }

        return $this->redirect(['action' => 'gateway', 'forceSecure' => true]);
    }

    /**
     * Redirects the user to Consors Finanz page.
     *
     * @throws Exception
     */
    public function gatewayAction()
    {
        if (!$this->getBasket()){
            $this->handleError(ErrorCode::NO_ORDER);
            return;
        }

        if (!$this->validateBillingCountry()) {
            $this->handleError(ErrorCode::BILLING_COUNTRY_NOT_VALID);
            return;
        }

        if (!$this->validateMinOrderAmount()) {
            $this->handleError(ErrorCode::ORDER_AMOUNT_TOO_LOW);
            return;
        }

        if (!$this->validateMaxOrderAmount()) {
            $this->handleError(ErrorCode::ORDER_AMOUNT_TOO_HIGH);
            return;
        }

        $user = $this->getUser();

        $financing = new Financing();
        $financing->setCustomerName($user['billingaddress']['firstname'] . ' ' . $user['billingaddress']['lastname']);
        $financing->setCustomerEmail($user['additional']['user']['email']);
        $financing->setAmount($this->getAmount());
        $financing->setBillingCountryIso($this->getCustomerBillingCountryIso());

        $financingRequest = $this->createFinancingRequest($financing);

        if ($this->settingsProvider->isCreateOrderBeforePayment()) {
            $paymentStatusId = $this->statusService->findStatusByName(PaymentStatus::ANALYSING)->getId();
            $transactionId = $financing->getUniqueId();

            $orderNumber = $this->saveOrder(
                $transactionId,
                $transactionId,
                $paymentStatusId
            );

            $financing->setOrderNumber($orderNumber);
            $financing->setTransactionId($transactionId);
            $financing->setStatus($paymentStatusId);

            $financingRequest->setOrderNumber($orderNumber);
        }

        $this->modelManager->persist($financing);
        $this->modelManager->flush();

        $this->logger->info('Financing request', $financingRequest->toArray(), $financing->getFinancingContext());

        $this->View()->assign('financingRequest', $financingRequest);
    }

    /**
     * Executed on asynchronous payment status updates.
     *
     * @throws Exception
     */
    public function notifyAction()
    {
        $this->Front()->Plugins()->ViewRenderer()->setNoRender();

        $request = $this->Request();

        if (!$request->isPost()) {
            return;
        }

        $this->restoreSession($request);

        $notification = $this->notificationService->createNotificationFromRequest($request);

        /** @var Financing $financing */
        $financing = $this->financingRepository->findOneBy(['uniqueId' => $notification->getUniqueId()]);

        if (!$financing) {
            return;
        }

        $this->logger->info('Notification', $request->getParams(), $financing->getFinancingContext());

        $hashKey = $this->settingsProvider->getHashKey($financing->getFinancingContext());

        if (!$this->notificationService->validateNotification($notification, $hashKey)) {
            return;
        }

        $financing = $this->financingService->updateFinancingFromNotification($financing, $notification);

        if (!$financing->hasOrder() && $notification->isStatusProposal()) {
            $this->createOrderOnProposalNotification($financing);
            return;
        }

        if ($financing->hasOrder()) {
            $this->updateOrderOnNotification($financing);
        }
    }

    /**
     * Executed if the credit was approved on the Consors Finanz page and the customer navigates back to the shop.
     *
     * @throws Exception
     */
    public function successAction()
    {
        $financingUniqueId = $this->Request()->getParam('uid');

        /** @var Financing $financing */
        $financing = $this->financingRepository->findOneBy(['uniqueId' => $financingUniqueId]);

        if (!$financing) {
            $this->handleError(ErrorCode::FINANCING_NOT_FOUND);
            return;
        }

        // It is possible that the order was already saved before the customer returned from the Consors Finanz
        // page to the successAction. It can be the case when (1) createOrderBeforePayment setting is active, or
        // (2) Consors Finanz sent a proposal notification. If not, the order is saved here with financing's unique ID
        // as a temporary transaction ID. The temporary ID will be replaced with the correct one returned from
        // Consors Finanz on notifyAction.
        if (!$financing->hasOrder()) {
            $statusId = $this->statusService->findStatusByName(PaymentStatus::WAITING_FOR_DOCUMENTS)->getId();

            $orderNumber = $this->saveOrder($financingUniqueId, $financingUniqueId, $statusId, true);

            $financing->setOrderNumber($orderNumber);
            $financing->setStatus($statusId);
            $financing->setUpdatedTimeToNow();

            $this->modelManager->persist($financing);
            $this->modelManager->flush();
        }

        $this->redirect([
            'controller' => 'checkout',
            'action' => 'finish',
            'sUniqueID' => $financingUniqueId
        ]);
    }

    /**
     * Executed if the customer cancels the payment process on the Consors Finanz page.
     *
     * @throws Exception
     */
    public function cancelAction()
    {
        $financingUniqueId = $this->Request()->getParam('uid');

        /** @var Financing $financing */
        $financing = $this->financingRepository->findOneBy(['uniqueId' => $financingUniqueId]);

        if (!$financing) {
            $this->handleError(ErrorCode::FINANCING_NOT_FOUND);
            return;
        }

        $statusId = $this->statusService->findStatusByName(PaymentStatus::CANCELLED)->getId();

        $financing->setStatus($statusId);
        $financing->setUpdatedTimeToNow();

        $this->modelManager->persist($financing);
        $this->modelManager->flush();

        // Update the status of the order if it was created before the payment (createOrderBeforePayment setting
        // is active)
        if ($financing->hasOrder()) {
            $this->savePaymentStatus($financing->getTransactionId(), $financing->getUniqueId(), $statusId, true);
        }

        $this->handleError(ErrorCode::PAYMENT_CANCELLED_BY_CUSTOMER);
    }

    /**
     * Executed if Consors Finanz rejects the credit request.
     *
     * @throws Exception
     */
    public function failureAction()
    {
        $financingUniqueId = $this->Request()->getParam('uid');

        /** @var Financing $financing */
        $financing = $this->financingRepository->findOneBy(['uniqueId' => $financingUniqueId]);

        if (!$financing) {
            $this->handleError(ErrorCode::FINANCING_NOT_FOUND);
            return;
        }

        $statusId = $this->statusService->findStatusByName(PaymentStatus::REFUSED)->getId();

        $financing->setStatus($statusId);
        $financing->setUpdatedTimeToNow();

        $this->modelManager->persist($financing);
        $this->modelManager->flush();

        // Update the status of the order if it was created before the payment (createOrderBeforePayment setting
        // is active)
        if ($financing->hasOrder()) {
            $this->savePaymentStatus($financing->getTransactionId(), $financing->getUniqueId(), $statusId, true);
        }

        $this->handleError(ErrorCode::FINANCING_REJECTED);
    }

    /**
     * {@inheritdoc}
     */
    public function getWhitelistedCSRFActions(): array
    {
        return ['notify'];
    }

    /**
     * @param Financing $financing
     *
     * @throws OptimisticLockException
     * @throws ORMException
     */
    private function createOrderOnProposalNotification(Financing $financing)
    {
        $orderNumber = $this->saveOrder(
            $financing->getTransactionId(),
            $financing->getUniqueId(),
            $financing->getStatus(),
            true
        );
        $financing->setOrderNumber($orderNumber);

        $this->modelManager->persist($financing);
        $this->modelManager->flush();

        $this->apiClient->updateFinancingOrderNumber($financing);
    }

    /**
     * @param Financing $financing
     *
     * @throws ORMException
     * @throws OptimisticLockException
     */
    private function updateOrderOnNotification(Financing $financing)
    {
        /** @var Order $order */
        $order = $this->orderRepository->findOneBy(['number' => $financing->getOrderNumber()]);

        if (!$order) {
            return;
        }

        // If the order was saved in the gatewayAction (createOrderBeforePayment setting is active), the transaction ID
        // of the order is temporary and equal to the unique ID of the financing, thus has to be updated with the correct
        // one provided by Consors Finanz.
        if ($order->getTransactionId() == $financing->getUniqueId()) {
            $order->setTransactionId($financing->getTransactionId());

            $this->modelManager->persist($order);
            $this->modelManager->flush();
        }

        // update the payment status of the order
        $this->savePaymentStatus(
            $financing->getTransactionId(),
            $financing->getUniqueId(),
            $financing->getStatus(),
            true
        );
    }

    /**
     * @param Financing $financing
     *
     * @return FinancingRequest
     */
    private function createFinancingRequest(Financing $financing): FinancingRequest
    {
        $user = $this->getUser();
        $billingAddress = $user['billingaddress'];
        $birthday = $user['additional']['user']['birthday'] ?: null;
        $phone = $billingAddress['phone'];
        $shopLogo = $this->settingsProvider->getShopLogo();
        $shopName = $this->settingsProvider->getShopName();
        $defaultDuration = $this->settingsProvider->getDefaultDuration();
        $financingUid = $financing->getUniqueId();

        $financingRequest = new FinancingRequest();
        $financingRequest->setVendorId($this->settingsProvider->getVendorId($financing->getFinancingContext()));
        $financingRequest->setAmount($this->getAmount());
        $financingRequest->setSalutation($billingAddress['salutation'] == 'mr' ? 'HERR' : 'FRAU');
        $financingRequest->setFirstName($billingAddress['firstname']);
        $financingRequest->setLastName($billingAddress['lastname']);
        $financingRequest->setStreet($billingAddress['street']);
        $financingRequest->setZip($billingAddress['zipcode']);
        $financingRequest->setCity($billingAddress['city']);
        $financingRequest->setEmail($user['additional']['user']['email']);
        $financingRequest->setGatewayUrl(Url::GATEWAY[$financing->getBillingCountryIso()]);
        $financingRequest->setCancelUrl($this->router->assemble([
            'action' => 'cancel',
            'forceSecure' => true,
            'uid' => $financingUid
        ]));
        $financingRequest->setFailureUrl($this->router->assemble([
            'action' => 'failure',
            'forceSecure' => true,
            'uid' => $financingUid
        ]));
        $financingRequest->setSuccessUrl($this->router->assemble([
            'action' => 'success',
            'forceSecure' => true,
            'uid' => $financingUid
        ]));
        $financingRequest->setNotifyUrl($this->router->assemble(array_merge(
            [
                'action' => 'notify',
                'forceSecure' => true,
                'uid' => $financingUid
            ],
            $this->getNotifyUrlSessionParam()
        )));

        if ($phone) {
            $financingRequest->setPhone($phone);
        }

        if ($birthday) {
            $financingRequest->setBirthdate(DateTime::createFromFormat('Y-m-d', $birthday));
        }

        if ($defaultDuration) {
            $financingRequest->setDefaultDuration($defaultDuration);
        }

        if ($shopLogo) {
            $financingRequest->setShopLogoUrl($shopLogo);
        }

        if ($shopName) {
            $financingRequest->setShopName($shopName);
        }

        if ($this->settingsProvider->getCampaignId() > 0
                && $this->campaignService->containsBasketOnlyCampaignArticles($this->getBasket()['content'])) {
            $financingRequest->setCampaignId($this->settingsProvider->getCampaignId());
            $financingRequest->setCampaignDuration($this->settingsProvider->getCampaignDuration());
        }

        return $financingRequest;
    }

    /**
     * @return bool
     */
    private function validateBillingCountry(): bool
    {
        $customerBillingCountryIso = $this->getCustomerBillingCountryIso();

        return in_array($customerBillingCountryIso, $this->settingsProvider->getValidCountryIsos());
    }

    /**
     * @return bool
     */
    private function validateMinOrderAmount(): bool
    {
        $amount = $this->getAmount();
        $minAmount = $this->settingsProvider->getMinOrderAmount();

        return $amount >= $minAmount;
    }

    /**
     * @return bool
     */
    private function validateMaxOrderAmount(): bool
    {
        $amount = $this->getAmount();
        $maxAmount = $this->settingsProvider->getMaxOrderAmount();

        return $amount <= $maxAmount;
    }

    /**
     * @param int $errorCode
     *
     * @throws Exception
     */
    private function handleError(int $errorCode)
    {
        $this->redirect([
            'controller' => 'checkout',
            'mille_consors_finanz_error' => $errorCode]
        );
    }

    /**
     * @return string
     */
    private function getCustomerBillingCountryIso(): string
    {
        $user = $this->getUser();

        if (!isset($user['billingaddress']['countryId'])) {
            return '';
        }

        /** @var Country $country */
        $country = $this->countryRepository->find($user['billingaddress']['countryId']);

        return $country ? $country->getIso() : '';
    }

    /**
     * @return array
     * @throws Exception
     */
    private function getNotifyUrlSessionParam(): array
    {
        // appendSession router param was removed in Shopware 5.5.7 and replaced with OptinService
        // session data is needed to create and update order on notification
        if (version_compare($this->shopwareConfig->offsetGet('version'), '5.5.7', '<')) {
            return ['appendSession' => true];
        }

        $hash = $this->get('shopware.components.optin_service')->add(self::OPTIN_SESSION_PARAM, 2592000, [
            'sessionId' => $this->session->get('sessionId'),
        ]);

        return ['sessionHash' => $hash];
    }

    /**
     * @param Enlight_Controller_Request_RequestHttp $request
     *
     * @throws Exception
     */
    private function restoreSession(Enlight_Controller_Request_RequestHttp $request)
    {
        $sessionHash = $request->getParam('sessionHash');

        if (!$sessionHash) {
            return;
        }

        $sessionData = $this->get('shopware.components.optin_service')->get(self::OPTIN_SESSION_PARAM, $sessionHash);

        if (!$sessionData) {
            throw new Exception('Session hash not found');
        }

        session_write_close();
        session_id($sessionData['sessionId']);
        session_start();
    }
}
