<?php

namespace MilleConsorsFinanz\Setup;

use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityRepository;
use Doctrine\ORM\Tools\SchemaTool;
use MilleConsorsFinanz\Components\PaymentStatus;
use MilleConsorsFinanz\Models\Financing\Financing;
use MilleConsorsFinanz\Models\Settings\Settings;
use Shopware\Components\Model\ModelManager;
use Shopware\Components\Plugin\Context\InstallContext;
use Shopware\Components\Plugin\PaymentInstaller;
use Shopware\Models\Order\Status;
use Shopware_Components_Config;

class Installer
{
    /**
     * @var InstallContext
     */
    private $installContext;

    /**
     * @var PaymentInstaller
     */
    private $paymentInstaller;

    /**
     * @var ModelManager
     */
    private $modelManager;

    /**
     * @var Connection
     */
    private $connection;

    /**
     * @var EntityRepository
     */
    private $statusRepository;

    /**
     * @var Shopware_Components_Config
     */
    private $shopwareConfig;

    /**
     * @param InstallContext $installContext
     * @param PaymentInstaller $paymentInstaller
     * @param ModelManager $modelManager
     * @param Connection $connection
     * @param Shopware_Components_Config $shopwareConfig
     */
    public function __construct(
        InstallContext $installContext,
        PaymentInstaller $paymentInstaller,
        ModelManager $modelManager,
        Connection $connection,
        Shopware_Components_Config $shopwareConfig
    ) {
        $this->installContext = $installContext;
        $this->paymentInstaller = $paymentInstaller;
        $this->modelManager = $modelManager;
        $this->connection = $connection;
        $this->shopwareConfig = $shopwareConfig;

        $this->statusRepository = $this->modelManager->getRepository(Status::class);
    }

    /**
     * @return bool
     */
    public function install()
    {
        $this->createModels();
        $this->createPaymentMethod();
        $this->createPaymentStatuses();
        $this->createCmsPage();

        return true;
    }

    private function createModels()
    {
        $schemaTool = new SchemaTool($this->modelManager);

        $schemaTool->updateSchema($this->getPluginModels(), true);
    }

    /**
     * @return array
     */
    private function getPluginModels(): array
    {
        return [
            $this->modelManager->getClassMetadata(Financing::class),
            $this->modelManager->getClassMetadata(Settings::class)
        ];
    }

    private function createPaymentMethod()
    {
        $options = [
            'name' => 'MilleConsorsFinanz',
            'description' => 'Finanzierung',
            'action' => 'MilleConsorsFinanz',
            'active' => 0,
            'position' => 0,
            'additionalDescription' =>
                '<img src="{link file=\'frontend/_public/src/img/Consors_Finanz_Logo.png\' fullPath}"> '
                . 'Gemeinsam mit unserem Finanzierungspartner, der Consors Finanz GmbH, bieten wir Ihnen eine bequeme '
                . 'Online-Finanzierung an.'
        ];

        $this->paymentInstaller->createOrUpdate($this->installContext->getPlugin(), $options);
    }

    private function createPaymentStatuses()
    {
        $statusesData = $this->getPaymentStatusesData();
        $statusId = $this->getHighestStatusId();

        foreach ($statusesData as $statusData) {
            $status = $this->statusRepository->findOneBy(['name' => $statusData['name']]);

            if (!$status) {
                $this->insertStatus(++$statusId, $statusData['name'], $statusData['description'], $statusData['mail']);
            }
        }
    }

    /**
     * @return int
     */
    private function getHighestStatusId(): int
    {
        return $this->connection->createQueryBuilder()
            ->select('*')
            ->from('s_core_states')
            ->orderBy('id', 'DESC')
            ->setMaxResults(1)
            ->execute()
            ->fetchColumn();
    }

    /**
     * @param int $id
     * @param string $name
     * @param string $description
     * @param int $mail
     */
    private function insertStatus(int $id, string $name, string $description, int $mail)
    {
        $this->connection->insert('s_core_states', [
            'id' => $id,
            'name' => $name,
            'description' => $description,
            'position' => $id,
            '`group`' => 'payment',
            'mail' => $mail
        ]);
    }

    /**
     * @return array
     */
    private function getPaymentStatusesData(): array
    {
        return [
            [
                'name' => PaymentStatus::REFUSED,
                'description' => 'Kreditanfrage wurde abgelehnt.',
                'mail' => 1
            ],
            [
                'name' => PaymentStatus::WAITING_FOR_DOCUMENTS,
                'description' => 'Kreditanfrage wurde vorläufig genehmigt.',
                'mail' => 1
            ],
            [
                'name' => PaymentStatus::ANALYSING,
                'description' => 'Kreditanfrage wird analysiert.',
                'mail' => 0
            ],
            [
                'name' => PaymentStatus::PROPOSAL,
                'description' => 'Kreditanfrage in Prüfung.',
                'mail' => 0
            ],
            [
                'name' => PaymentStatus::ACCEPTED,
                'description' => 'Kreditanfrage wurde genehmigt.',
                'mail' => 0
            ],
            [
                'name' => PaymentStatus::DELIVERED,
                'description' => 'Die Lieferbestätigung wurde übermittelt.',
                'mail' => 0
            ],
            [
                'name' => PaymentStatus::CANCELLED,
                'description' => 'Kreditanfrage wurde storniert.',
                'mail' => 0
            ]
        ];
    }

    private function createCmsPage()
    {
        $pageExists = $this->connection->fetchColumn(
            'SELECT COUNT(*) as total FROM s_cms_static WHERE description = "Finanzierung"'
        );

        if ($pageExists) {
            return;
        }

        // groups were renamed in Shopware 5.5.0
        $groups = version_compare($this->shopwareConfig->offsetGet('version'), '5.5.0', '<') ?
            'gLeft|gBottom2' : 'left|bottom2';

        $financingPageContent =
            '<div class="financing--description">
                <p>Bezahlen Sie schnell und unkompliziert in kleinen Monatsraten. Einfach Artikel auswählen, zur Kasse gehen und die Zahlungsart "Finanzierung" auswählen. Die Vermittlung erfolgt ausschließlich für den Kreditgeber BNP Paribas S.A. Niederlassung Deutschland, Standort München: Schwanthalerstr. 31, 80336 München.</p>
                <h2>Finanzierungsprozess</h2>
                <p>Sie haben zwei Möglichkeiten, die Legitimation und Unterschrift vorzunehmen:<br>
                    Komplett online (mit sofortigem Warenversand) oder schriftlich und über eine Postfiliale (Warenversand nach Eingang der Unterlagen).</p>
                <p><u>So funktioniert' . "''" . 's:</u></p>
                <ol>
                    <li><strong>Finanzierung online beantragen:</strong> Füllen Sie online das Antragsformular aus.</li>
                    <li><strong>Legitimation:</strong><br>
                        Möglichkeit 1:Lassen Sie Ihre Identität bequem online per Videotelefonie bestätigen.<br>
                        Möglichkeit 2: Lassen Sie Ihre Identität in der nächsten Postfiliale per Postident bestätigen.</li>
                    <li><strong>Vertrag unterschreiben:</strong><br>
                        Möglichkeit 1: Unterschreiben Sie den Kreditvertrag online per mTAN (SMS).<br>
                        Möglichkeit 2: Drucken Sie den Kreditvertrag aus, unterschreiben handschriftlich und senden ihn per Post ein.</li>
                    <li><strong>Auf die Lieferung freuen:</strong> Ihre bestellte Ware liefern wir unverzüglich an Sie aus, sobald Ihr Finanzierungsantrag bei unserer Partnerbank eingegangen ist (vorbehaltlich einer abschließenden positiven Prüfung nach Eingang Ihres Antrags). Haben Sie den Antrag online eingereicht, kann die Ware sofort versendet werden.</li>
                </ol>

                <h2>Einzureichende Unterlagen</h2>
                <p><u>Folgende Unterlagen benötigt die Bank:</u></p>
                <ul class="list--unordered">
                    <li>Unterschriebener Kreditvertrag (elektronisch oder per Post)</li>
                    <li>Letzter Gehaltsnachweis / bei Selbständigen die letzte BWA (bei einer Finanzierungssumme ab 4.000 €; in Einzelfällen auch unter einer Finanzierunssumme von 4.000 €)</li>
                    <li>Eine Kopie der Aufenthaltserlaubnis bei Staatsangehörigen von Ländern außerhalb der EU</li>
                    <li>Studenten: Kopie eines gültigen Studentenausweises bzw. Immatrikulationsbescheinigung für das laufende Semester und Kopie der ec-/Maestro- oder Kreditkarte</li>
                    <li>Auszubildende: Kopie einer aktuellen Gehaltsabrechnung und Kopie der ec-/Maestro- oder Kreditkarte</li>
                </ul>

                <h2>Voraussetzungen für eine Finanzierung</h2>
                <p><u>Folgende Voraussetzungen hinsichtlich Beschäftigung / Einkommen müssen erfüllt sein:</u></p>
                <ul class="list--unordered">
                    <li>Sie sind mindestens 18 Jahre alt</li>
                    <li>Die Finanzierungssumme liegt zwischen 100 € und 15.000 €<br>
                        <i>(Hinweis: Theoretisch ist eine Finanzierung bereits ab 54 € möglich. In diesem Fall müssen bei Ihnen 6 Monate Lauzeit auswählbar sein.)</i></li>
                    <li>Sie sind Angestellter und arbeiten bei Ihrem derzeitigen Arbeitgeber seit mind. 3 Monaten.</li>
                    <li>Ihr Arbeitsverhältnis ist für die nächsten 12 Monate unbefristet / oder Sie sind Rentner mit einer unbefristet gewährten Rente / oder Sie sind seit mindestens zwölf Monaten selbständig</li>
                    <li>Sie haben ein regelmäßiges Einkommen von mind. 650 € netto</li>
                    <li>Sie haben keinen negativen Schufa-Eintrag</li>
                </ul>

                <h2>Legitimation</h2>
                <p>Wie alle Banken ist auch Consors Finanz gesetzlich dazu verpflichtet, eine Legitimationsprüfung durchzuführen. Dabei wird die Identität aller neuen Kunden vor einer Kreditvergabe überprüft. Diese Prüfung dient auch Ihrer eigenen Sicherheit. Sie haben die Wahl, ob Sie die Legitimation <strong>online per Videotelefonie</strong> (mit PC, Tablet oder Smartphone) oder in der nächsten Filiale der Deutschen Post <strong>per PostIdent</strong> durchführen möchten. Auch die Vertragsunterschrift kann entweder online <strong>mit elektronischer Unterschrift</strong> erfolgen oder Sie <strong>unterschreiben handschriftlich und die Deutsche Post sendet den Vertrag per Post</strong> an uns. Bitte beachten Sie, dass das Einsenden des Vertrags per Post einige Tage zusätzlich benötigt.</p>

                <h2>Finanzierungsabschluss mit PostIdent</h2>
                <p><u>Legitimation mit PostIdent – So funktionierts:</u></p>
                <ul class="list--unordered">
                    <li><strong>Entscheiden Sie sich für die Zahlart "Finanzierung":</strong> Nachdem Sie Ihre Ware in den Warenkorb gelegt haben, können Sie sich beim Gang zur Kasse für eine Finanzierung entscheiden. Sie durchlaufen den kurzen Online-Antragsprozess von Consors Finanz. Entscheiden Sie sich nach der vorläufigen Kreditgenehmigung dafür, den Vertrag auszudrucken und zur Post zu bringen.</li>
                    <li><strong>Legitimieren Sie sich bei einer Postfiliale:</strong> Drucken Sie Ihren Antrag aus und lesen sich Ihre Unterlagen in Ruhe durch. Unterschreiben Sie den Vertrag an den vorgesehenen Stellen und legen Sie die von der Bank geforderten Unterlagen bei. Verschließen Sie die Unterlagen in einem Umschlag und bringen Sie diesen mit dem Postident-Formular zur nächsten Postfiliale. Ein Mitarbeiter der Post bestätigt Ihre Identität anhand Ihres gültigen Personalausweises oder Reisepasses.</li>
                    <li><strong>Freuen Sie sich auf die Lieferung:</strong> Nach Eingang des Finanzierungsantrags bei Consors Finanz liefern wir die von Ihnen bestellte Ware unverzüglich an Sie aus (vorbehaltlich einer abschließenden positiven Prüfung nach Eingang Ihrer Unterlagen).</li>
                </ul>

                <h2>Volldigitaler Finanzierungsabschluss</h2>
                <p><u>Volldigitaler Finanzierungsabschluss – So funktionierts:</u></p>
                <ul class="list--unordered">
                    <li><strong>Entscheiden Sie sich für die Zahlart "Finanzierung":</strong> Nachdem Sie Ihre Ware in den Warenkorb gelegt haben, können Sie sich beim Gang zur Kasse für eine Finanzierung entscheiden. Sie durchlaufen den kurzen Online-Antragsprozess von Consors Finanz. Entscheiden Sie sich nach der vorläufigen Finanzierungsbewilligung für die Option "Jetzt online abschließen", um den Antrag digital abzuschließen.</li>
                    <li><strong>Online-Identifikation:</strong> Ihre Identifikation erfolgt ganz bequem mit einem Videotelefonat – einfach und direkt über Ihren Browser oder per App. Mitarbeiter führen Sie Schritt für Schritt durch den Prozess.</li>
                    <li><strong>Digitale Signatur:</strong> Im Anschluss an die Online-Identifikation erhalten Sie eine mTAN per SMS, die Sie zur Bestätigung in das dafür vorgesehen Feld eintragen. Damit wird der Vertrag rechtskräftig online unterschrieben und abgeschlossen.</li>
                    <li><strong>Digitaler Kontoblick:</strong> Dieser ersetzt das Einreichen von Kontoauszügen und Gehaltsnachweisen in Papierform. Die Kreditentscheidung und Prüfung Ihrer Angaben erfolgt mit einem einmaligen Blick auf Ihr (Gehalts-) Girokonto. So weisen Sie Ihr regelmäßiges Einkommen nach. Der Kontoblick ermöglicht uns die Auswertung der gelieferten Umsatzdaten, um eine Kreditentscheidung treffen zu können.</li>
                    <li><strong>Freuen Sie sich auf die baldige Lieferung:</strong> Nach dem Online-Abschluss kann Ihre Ware im Gegensazu zum PostIdent-Verfahren sofort versendet werden!</li>
                </ul>
            </div>';

        $this->connection->executeQuery("
            INSERT INTO s_cms_static (description, html, `grouping`, position, tpl1variable, tpl1path, tpl2variable, tpl2path, tpl3variable, tpl3path, link, target, page_title, meta_keywords, meta_description) 
            VALUES ('Finanzierung', '" . $financingPageContent . "', '" . $groups . "', '0', '', '', '', '', '', '', '', '', '', '', '');"
        );
    }
}
