<?php

namespace MilleConsorsFinanz\Subscriber\Frontend;

use Enlight\Event\SubscriberInterface;
use Enlight_Controller_ActionEventArgs;
use Enlight_View_Default;
use MilleConsorsFinanz\Components\Services\InstallmentsCalculator;
use MilleConsorsFinanz\Components\Settings\SettingsProvider;
use MilleConsorsFinanz\Struct\FinancingOption;

class Detail implements SubscriberInterface
{
    /**
     * @var InstallmentsCalculator
     */
    private $installmentsCalculator;

    /**
     * @var SettingsProvider
     */
    private $settingsProvider;

    /**
     * @param InstallmentsCalculator $installmentsCalculator
     * @param SettingsProvider $settingsProvider
     */
    public function __construct(
        InstallmentsCalculator $installmentsCalculator,
        SettingsProvider $settingsProvider
    ) {
        $this->installmentsCalculator = $installmentsCalculator;
        $this->settingsProvider = $settingsProvider;
    }

    /**
     * {@inheritdoc}
     */
    public static function getSubscribedEvents(): array
    {
        return [
            'Enlight_Controller_Action_PostDispatchSecure_Frontend_Detail' => [
                ['addFinancingExample'],
                ['addCalculator'],
                ['addInstallmentsTable'],
            ]
        ];
    }

    /**
     * @param Enlight_Controller_ActionEventArgs $args
     */
    public function addFinancingExample(Enlight_Controller_ActionEventArgs $args)
    {
        $controller = $args->getSubject();

        /** @var Enlight_View_Default $view */
        $view = $controller->View();

        $sArticle = $view->getAssign('sArticle');

        if (!$sArticle) {
            return;
        }

        $financingExample = $this->installmentsCalculator->getFinancingExample($sArticle['articleID'],
            $sArticle['price_numeric']);

        if (!$financingExample) {
            return;
        }

        $view->assign('milleConsorsFinanzFinancingExample', $financingExample->toArray());
    }

    /**
     * @param Enlight_Controller_ActionEventArgs $args
     */
    public function addCalculator(Enlight_Controller_ActionEventArgs $args)
    {
        $controller = $args->getSubject();

        /** @var Enlight_View_Default $view */
        $view = $controller->View();

        $sArticle = $view->getAssign('sArticle');

        if (!$sArticle) {
            return;
        }

        /** @var FinancingOption[] $financingOptions */
        $financingOptions = $this->installmentsCalculator->getFinancingOptions($sArticle['articleID'],
            $sArticle['price_numeric']);

        if (count($financingOptions) < 2) {
            return;
        }

        $minMonth = INF;
        $maxMonth = 0;
        $campaignDuration = 0;
        $campaignInterestRate = $this->settingsProvider->getCampaignInterestRate();
        $interestRate = 0;

        foreach ($financingOptions as $financingOption) {
            $duration = $financingOption->getDuration();

            if ($duration < $minMonth) {
                $minMonth = $duration;
            }

            if ($duration > $maxMonth) {
                $maxMonth = $duration;
            }

            if ($financingOption->getInterestRate() == $campaignInterestRate) {
                if ($duration > $campaignDuration) {
                    $campaignDuration = $duration;
                }
            } else {
                $interestRate = $financingOption->getInterestRate();
            }
        }

        $stepMonth = abs($financingOptions[1]->getDuration() - $financingOptions[0]->getDuration());

        $view->assign('milleConsorsFinanzCalculatorData', [
            'minMonth' => $minMonth,
            'maxMonth' => $maxMonth,
            'stepMonth' => $stepMonth,
            'interestRate' => $interestRate,
            'campaignDuration' => $campaignDuration,
            'campaignInterestRate' => $campaignInterestRate,
            'productPrice' => $sArticle['price_numeric'],
        ]);
    }

    /**
     * @param Enlight_Controller_ActionEventArgs $args
     */
    public function addInstallmentsTable(Enlight_Controller_ActionEventArgs $args)
    {
        $controller = $args->getSubject();

        /** @var Enlight_View_Default $view */
        $view = $controller->View();

        $sArticle = $view->getAssign('sArticle');

        if (!$sArticle) {
            return;
        }

        /** @var FinancingOption[] $financingOptions */
        $financingOptions = $this->installmentsCalculator->getFinancingOptions($sArticle['articleID'],
            $sArticle['price_numeric']);

        if (!$financingOptions) {
            return;
        }

        $financingOptionsAsArray = [];

        foreach ($financingOptions as $financingOption) {
            $financingOptionsAsArray[] = $financingOption->toArray();
        }

        $view->assign('milleConsorsFinanzInstallments', $financingOptionsAsArray);
    }
}
