<?php
namespace MilleConsorsFinanzRK\Components\Consors\Services;

use MilleConsorsFinanzRK\Components\Logger\LoggerServiceInterface;
use Shopware\Components\Cart\Struct\Price;
use Shopware\Components\DependencyInjection\Container as DIContainer;
use Shopware\Components\Model\ModelManager;
use MilleConsorsFinanzRK\Components\Configuration;
use MilleConsorsFinanzRK\Components\Consors\Api\ConsorsSoapClient;
use MilleConsorsFinanzRK\Components\DependencyProvider;
use MilleConsorsFinanzRK\Components\Logger\LoggerService;
use MilleConsorsFinanzRK\Components\Services\ConsorsService;
use Shopware\Models\Shop\Shop;
use Shopware\Models\Shop\Repository as ShopRepository;

abstract class AbstractService
{
    protected $configData;

    /**
     * @var ConsorsSoapClient
     */
    protected $consorsApi;

    /**
     * @var DependencyProvider
     */
    protected $dependencyProvider;

    protected $orderData;

    /**
     * @var DIContainer
     */
    protected $container;
    /**
     * @var ModelManager
     */
    protected $modelManager;
    /**
     * @var LoggerService
     */
    protected $loggerService;
    /**
     * @var ConsorsService
     */
    protected $consorsService;
    /**
     * @var Configuration
     */
    private $configuration;

    /**
     * BoniCheck2Service constructor.
     * @param Configuration          $configuration
     * @param DIContainer            $container
     * @param DependencyProvider     $dependencyProvider
     * @param ModelManager           $modelManager
     * @param LoggerServiceInterface $loggerService
     * @param ConsorsService         $consorsService
     * @throws \SoapFault
     */
    public function __construct(
        Configuration $configuration,
        DIContainer $container,
        DependencyProvider $dependencyProvider,
        ModelManager $modelManager,
        LoggerServiceInterface $loggerService,
        ConsorsService $consorsService
    ) {
        $this->container = $container;
        $this->modelManager = $modelManager;

        $this->configData = $configuration->getPluginData();
        $this->dependencyProvider = $dependencyProvider;

        $this->registerShopResource();

        $this->consorsApi = new ConsorsSoapClient($container);

        $session = $this->dependencyProvider->getSession();
        $this->orderData = $session['sOrderVariables'];
        $this->loggerService = $loggerService;
        $this->consorsService = $consorsService;
        $this->configuration = $configuration;
    }

    /**
     * @param $changetyp
     * @param $shopId
     * @return array
     */
    protected function getDefaultParams($changetyp, $shopId)
    {
        return [
            'config' => $this->getConfigByShopId($shopId),
            'anfrageart' => $changetyp,
            'terminalclientaddress' => Shopware()->Front()->Request()->getClientIp(),
            'userreference' => $this->getUserReference(),
            'bestellwert' => round($this->orderData['sBasket']['sAmount'] * 100, 2)
        ];
    }

    protected function getConfigByShopId($shopId = 0)
    {
        $shopModel = $this->modelManager->getRepository(Shop::class)->find((int)$shopId);

        if ($shopModel) {
            return $this->configuration->getPluginData($shopModel);
        }

        return [];
    }

    protected function getUserReference()
    {
        $session = Shopware()->Session();
        if (empty($session['consorsUserReference'])) {
            $consorsUserReference = $this->consorsService->getRandomString(9);
            $session['consorsUserReference'] = $consorsUserReference;
        }
        return $session['consorsUserReference'];
    }

    protected function getAddress($address)
    {
        return [
            'strasse' => $address['street'],
            'plz' => ($address['zipcode'] != null) ? $address['zipcode'] : $address['zipCode'],
            'ort' => $address['city'],
            'land' => $this->getCountryIso($address['countryId']),
        ];
    }

    /**
     * @return array
     */
    public function getBasket()
    {
        $basket = $this->dependencyProvider->getBasket()->sGetBasket();

        if (!Shopware()->System()->sUSERGROUPDATA['tax'] && Shopware()->System()->sUSERGROUPDATA['id']) {
            $basket['sTaxRates'] = $this->getTaxRates($basket);
            $basket['sAmount'] = round($basket['AmountNetNumeric'], 2);
            $basket['sAmountTax'] = round($basket['AmountWithTaxNumeric'] - $basket['AmountNetNumeric'], 2);
            $basket['sAmountWithTax'] = round($basket['AmountWithTaxNumeric'], 2);
        } else {
            $basket['sTaxRates'] = $this->getTaxRates($basket);
            $basket['sAmount'] = $basket['AmountNumeric'];
            $basket['sAmountTax'] = round($basket['AmountNumeric'] - $basket['AmountNetNumeric'], 2);
        }

        return $basket;
    }

    /**
     *
     * @param array $basket array returned from this->getBasket
     *
     * @return array
     */
    public function getTaxRates($basket)
    {
        $result = [];

        if (!empty($basket['sShippingcostsTax'])) {
            if (!empty($basket['sShippingcostsTaxProportional'])) {
                /** @var Price $shippingTax */
                foreach ($basket['sShippingcostsTaxProportional'] as $shippingTax) {
                    $result[number_format($shippingTax->getTaxRate(), 2)] += $shippingTax->getTax();
                }
            } else {
                $basket['sShippingcostsTax'] = number_format((float) $basket['sShippingcostsTax'], 2);

                $result[$basket['sShippingcostsTax']] = $basket['sShippingcostsWithTax'] - $basket['sShippingcostsNet'];
                if (empty($result[$basket['sShippingcostsTax']])) {
                    unset($result[$basket['sShippingcostsTax']]);
                }
            }
        }

        if (empty($basket['content'])) {
            ksort($result, SORT_NUMERIC);

            return $result;
        }

        foreach ($basket['content'] as $item) {
            if (!empty($item['tax_rate'])) {
            } elseif (!empty($item['taxPercent'])) {
                $item['tax_rate'] = $item['taxPercent'];
            } elseif ($item['modus'] == 2) {
                $resultVoucherTaxMode = Shopware()->Db()->fetchOne(
                    'SELECT taxconfig FROM s_emarketing_vouchers WHERE ordercode=?',
                    [$item['ordernumber']]
                );

                $tax = null;

                // Old behaviour
                if (empty($resultVoucherTaxMode) || $resultVoucherTaxMode === 'default') {
                    $tax = Shopware()->Config()->get('sVOUCHERTAX');
                } elseif ($resultVoucherTaxMode === 'auto') {
                    // Automatically determinate tax
                    $tax = $this->dependencyProvider->getBasket()->getMaxTax();
                } elseif ($resultVoucherTaxMode === 'none') {
                    // No tax
                    $tax = '0';
                } elseif ((int) $resultVoucherTaxMode) {
                    // Fix defined tax
                    $tax = Shopware()->Db()->fetchOne('
                    SELECT tax FROM s_core_tax WHERE id = ?
                    ', [$resultVoucherTaxMode]);
                }
                $item['tax_rate'] = $tax;
            } else {
                $taxAutoMode = Shopware()->Config()->get('sTAXAUTOMODE');
                if (!empty($taxAutoMode)) {
                    $tax = $this->dependencyProvider->getBasket()->getMaxTax();
                } else {
                    $tax = Shopware()->Config()->get('sDISCOUNTTAX');
                }
                $item['tax_rate'] = $tax;
            }

            if (empty($item['tax_rate']) || empty($item['tax'])) {
                continue;
            } // Ignore 0 % tax

            $taxKey = number_format((float) $item['tax_rate'], 2);

            $result[$taxKey] += str_replace(',', '.', $item['tax']);
        }

        ksort($result, SORT_NUMERIC);

        return $result;
    }

    /**
     * @param int $countryId
     * @return bool|string
     */
    protected function getCountryIso($countryId = 0)
    {
        return $this->modelManager->getConnection()->createQueryBuilder()
            ->select('countryiso')
            ->from('s_core_countries')
            ->where('id = :id')
            ->setParameter('id', $countryId)
            ->setMaxResults(1)
            ->execute()
            ->fetchColumn();
    }

    protected function getUserBankData($userId)
    {
        return $this->modelManager->getConnection()->createQueryBuilder()
            ->select('*')
            ->from('s_core_payment_data')
            ->where('user_id = :id')
            ->setParameter('id', $userId)
            ->execute()
            ->fetch();
    }

    protected function translateSalutation($salutation)
    {
        $result = '';
        switch ($salutation) {
            case 'mr':
                $result = 'Herr';
                break;
            case 'ms':
                $result = 'Frau';
                break;
        }
        return $result;
    }

    private function registerShopResource()
    {
        /** @var ShopRepository $shopRepository */
        $shopRepository = $this->modelManager->getRepository(Shop::class);

        $query = $shopRepository->getBaseListQuery();
        $shops = $query->getArrayResult();

        foreach ($shops as $shop) {
            if (!$shop['active']) {
                continue;
            }

            $shopObject = $shopRepository->getActiveById($shop['id']);
            if ($shopObject === null) {
                $shopObject = $shopRepository->getActiveDefault();
            }

            if ($this->container->has('shopware.components.shop_registration_service')) {
                $this->container->get('shopware.components.shop_registration_service')->registerResources($shopObject);
            } else {
                $shopObject->registerResources();
            }
        }
    }

    private function registerShopResourceOld()
    {
        $query = $this->modelManager->getRepository(Shop::class)->getBaseListQuery();
        $shops = $query->getArrayResult();

        foreach ($shops as $shop) {
            if (!$shop['active']) {
                continue;
            }

            $frontendShop = $this->dependencyProvider->getShop();

            if (null == $frontendShop) {
                $this->bootstrapShop($shop['id']);
            } else {
                if ($shop['id'] == $frontendShop->getId()) {
                    $this->bootstrapShop($shop['id']);
                }
            }
        }
    }

    /**
     * @param $shopId
     * @return bool
     */
    private function bootstrapShop($shopId)
    {
        /* @var Shop $shopObject */
        $shopObject = $this->modelManager->getRepository(Shop::class)->find($shopId);

        if (!$shopObject) {
            return false;
        }

        if ($this->container->has('shopware.components.shop_registration_service')) {
            $this->container->get('shopware.components.shop_registration_service')->registerResources($shopObject);
        } else {
            $shopObject->registerResources();
        }

        return true;
    }

    function dumb($data)
    {
        highlight_string("<?php\n " . var_export($data, true) . "?>");
        echo '<script>document.getElementsByTagName("code")[0].getElementsByTagName("span")[1].remove() ;document.getElementsByTagName("code")[0].getElementsByTagName("span")[document.getElementsByTagName("code")[0].getElementsByTagName("span").length - 1].remove() ; </script>';
        die();
    }
}