<?php
namespace MilleConsorsFinanzRK\Components\Consors\Services;

use MilleConsorsFinanzRK\Bootstrap\Database;
use MilleConsorsFinanzRK\Components\Consors\Api\ConsorsSoapClientFault;
use MilleConsorsFinanzRK\Components\Consors\Request\Change2Request;
use MilleConsorsFinanzRK\Components\Consors\Response\Change2Response;
use MilleConsorsFinanzRK\Components\SettingsTable;
use Shopware\Models\Order\Document\Document;

class ChangeService extends AbstractService
{
    /**
     * @param array $document
     * @param       $changetyp
     * @param       $shopId
     * @param bool  $isGutschrift
     * @return Change2Response
     * @throws ConsorsSoapClientFault
     */
    public function change(array $document, $changetyp, $shopId, $isGutschrift = false)
    {
        $request = $this->getParams($document, $changetyp, $shopId, $isGutschrift);

        /** @var Change2Response $response */
        $response = $this->consorsApi->changeOrder($request);
        return $response;
    }

    /**
     * @param array $document
     * @param       $changetyp
     * @param       $shopId
     * @param bool  $isGutschrift
     * @return Change2Request
     */
    protected function getParams(array $document, $changetyp, $shopId, $isGutschrift = false)
    {
        $params = array_merge(
            $this->getDefaultParams($changetyp, $shopId),
            $this->getChangeParams($document, $isGutschrift)
        );

        return new Change2Request($params);
    }

    /**
     * @param $changetyp
     * @param $shopId
     * @return array
     */
    protected function getDefaultParams($changetyp, $shopId)
    {
        return [
            'config' => $this->getConfigByShopId($shopId),
            'changetyp' => $changetyp
        ];
    }

    /**
     * @param array $document
     * @param bool  $isGutschrift
     * @return \string[][]
     */
    protected function getChangeParams(array $document, $isGutschrift = false)
    {
        $belege = $this->getBelege($document, $isGutschrift);
        $debitor = $this->getDebitor($document);

        return [
            'referenzen' => [
                'referenztyp' => 'bestellnummer',
                'referenzwert' => $document['order']['number'],
            ],
            'debitor' => $debitor,
            'belege' => $belege,
        ];
    }

    protected function getDebitor($orderData)
    {
        $billing = $orderData['order']['billing'];
        $customer = $orderData['order']['customer'];
        return [
            'kundennummer' => $customer['number'],
            'bezeichnung' => $customer['firstname'] . ' ' . $customer['lastname'],
            'person' => $this->getPerson($customer, $billing)
        ];
    }

    protected function getBelege($orderData, $isGutschrift = false)
    {
        $order = $orderData['order'];
        $customer = $order['customer'];
        $payment = $order['payment'];

        $belegreferenz = (true == $isGutschrift) ? $this->getBelegReferenz($orderData) : $orderData['documentId'];

        if ('' == $belegreferenz) {
            $belegreferenz = $orderData['documentId'];
        }

        $beleg = [
            'kundennummer' => $customer['number'],
            'belegtyp' => (true == $isGutschrift) ? SettingsTable::BELEG_TYPE_GUTSCHRIFT : SettingsTable::BELEG_TYPE_RECHNUNG,
            'belegnummer' => $orderData['documentId'],
            'belegdatum' => $orderData['date']->format('Y-m-d'),
            'faellig' => date('Y-m-d', strtotime('+30 days',strtotime($orderData['date']->format('Y-m-d')))),
            'waehrung' => $order['currency'],
            'betrag' => (true == $isGutschrift) ? $orderData['amount'] * -1 : $orderData['amount'],
            'betrag_netto' => (true == $isGutschrift) ? $orderData['amount'] * -1 : $orderData['amount'],
            'zahlart' => (Database::DEBIT_PAYMENT_METHOD_NAME == $payment['name']) ? 'LA' : 'UEB',
            'rgbezahlt' => 'NEIN',
            'belegtext' => (Database::DEBIT_PAYMENT_METHOD_NAME == $payment['name']) ? 'Lastschrift' : 'Ueberweisung',
            'belegpositionen' => $this->getBelegPositions($orderData['order']),
        ];

        if ($isGutschrift) {
            $beleg['belegreferenz'] = $belegreferenz;
            $beleg['belegtext'] = 'Gutschrift zu Rechnung ' . $belegreferenz . ' vom ' . $orderData['date']->format('d.m.Y');
        } else {
            $beleg['belegtext'] = 'Rechnung zur Bestellung ' . $order['number'] . ' vom ' . $orderData['date']->format('d.m.Y');
        }

        return $beleg;
    }

    protected function getBelegReferenz($document)
    {
        $criteria = [
            'typeId' => 1,
            'orderId' => $document['order']['id'],
            'customerId' => $document['order']['customer']['id']
        ];

        /** @var Document $document */
        $document = $this->modelManager->getRepository(Document::class)
            ->findOneBy($criteria);

        if ($document) {
            return $document->getDocumentId();
        }

        return '';
    }

    protected function getBelegPositions($orderData, $isManagementSystem = false)
    {
        $customer = $orderData['customer'];
        $details = $orderData['details'];
        $attribute = $orderData['attribute'];

        $result = $articles = [];
        $key = 1;

        foreach ($details as $detail) {
            $taxRate = $detail['taxRate'];
            $price = $detail['price'];
            $quantity = $detail['quantity'];
            $result[] = [
                'kundennummer' => (true == $isManagementSystem && '' != $attribute['consorsFinanzCustomerNumber']) ? $attribute['consorsFinanzCustomerNumber'] : $customer['number'],
                'belegpositionsnummer' => $key,
                'betrag' => $price * $quantity,
                'betrag_netto' => $this->getNetto($price * $quantity, $taxRate),
                'umsatzsteuersatz' => round($taxRate * 100, 2),
                'waehrung' => $orderData['currency'],
                'artikel' => [
                    'menge' => $quantity,
                    'einheit' => ($detail['unit'] != 'Stück') ? $detail['unit'] : '',
                    'artikelnummer' => $detail['articleNumber'],
                    'artikelbezeichnung' => $detail['articleName'],
                    'artikelzusatz' => '',
                    'betrag' => $price,
                    'betrag_netto' => $this->getNetto($price, $taxRate),
                    'umsatzsteuersatz' => round($taxRate * 100, 2),
                    'warengruppe' => '',
                ]
            ];

            $key++;
        }

        return $result;
    }

    protected function getNetto($brutto, $taxpercent)
    {
        $b = doubleval($brutto);
        $p = doubleval($taxpercent);
        if ($p <= 0) {
            return $b;
        }
        return (($b)/(1+($p/100)));
    }

    protected function getPerson($user, $address, $liefer = false)
    {
        $firstName = ($address['firstname'] != null) ? $address['firstname'] : $user['firstName'];
        $lastName = ($address['lastname'] != null) ? $address['lastname'] : $user['lastName'];

        $person = [
            'salutation' => $this->translateSalutation($address['salutation']),
            'vorname' => ($liefer === true) ? $firstName : $user['firstname'],
            'name' => ($liefer === true) ? $lastName : $user['lastname'],
            'geburtsdatum' => (is_string($user['birthday'])) ? $user['birthday'] : $user['birthday']->format('Y-m-d'),
            'hauptanschrift' => [
                'adresse' => $this->getAddress($address),
                'zusatz' => $address['additionalAddressLine1']
            ],
            'kontakte' => [
                [
                    'typ' => 'mobil',
                    'kontakt' => $address['phone']
                ],
                [
                    'typ' => 'email',
                    'kontakt' => $user['email']
                ]
            ],
            'rechtsform' => '',
            'handelsregisternummer' => '',
            'amtsgericht' => '',
        ];

        $bankData = $this->getUserBankData($user['id']);
        if (!empty($bankData)) {
            $person = array_merge(
                $person,
                ['bankverbindungen' => $bankData]
            );
        }
        return $person;
    }

    protected function getTaxFromGrossAndNet($gross, $net)
    {
        return round((($gross - $net) / $net) * 100, 2);
    }

    protected function convertDate($originalDate)
    {
        if ('' == $originalDate) {
            return '';
        }

        return date("Y-m-d", strtotime($originalDate));
    }
}