<?php
namespace MilleConsorsFinanzRK\Components\Services;

use Doctrine\DBAL\DBALException;
use MilleConsorsFinanzRK\Components\Consors\Services\ChangeManagementSystemService;
use MilleConsorsFinanzRK\Components\Consors\Services\ChangeService;
use MilleConsorsFinanzRK\Components\Consors\Services\OrderStornoService;
use MilleConsorsFinanzRK\Components\Logger\LoggerServiceInterface;
use MilleConsorsFinanzRK\Components\SettingsTable;
use Shopware\Bundle\AttributeBundle\Service\DataPersister;
use Shopware\Components\Model\ModelManager;
use Shopware\Models\Order\Document\Document;
use Shopware\Models\Order\Order;
use Shopware\Models\Order\Status;
use Shopware\Models\Shop\Shop;

class CronService
{
    /**
     * @var ModelManager
     */
    private $modelManager;
    /**
     * @var LoggerServiceInterface
     */
    private $loggerService;

    /**
     * @var OrderStornoService
     */
    private $orderStornoService;

    /**
     * @var ChangeService
     */
    private $changeService;
    /**
     * @var DataPersister
     */
    private $dataPersister;

    /**
     * @var ChangeManagementSystemService
     */
    private $changeManagementSystemService;

    const INVOICE = 'Rechnung';

    const CANCELLATION = 'Stornorechnung';

    const GUTSCHRIFT = 'Gutschrift';

    /**
     * CronService constructor.
     * @param ModelManager                  $modelManager
     * @param LoggerServiceInterface        $loggerService
     * @param OrderStornoService            $orderStornoService
     * @param ChangeService                 $changeService
     * @param ChangeManagementSystemService $changeManagementSystemService
     * @param DataPersister        $dataPersister
     */
    public function __construct(
        ModelManager $modelManager,
        LoggerServiceInterface $loggerService,
        OrderStornoService $orderStornoService,
        ChangeService $changeService,
        ChangeManagementSystemService $changeManagementSystemService,
        DataPersister $dataPersister
    ) {
        $this->modelManager = $modelManager;
        $this->loggerService = $loggerService;
        $this->orderStornoService = $orderStornoService;
        $this->changeService = $changeService;
        $this->dataPersister = $dataPersister;
        $this->changeManagementSystemService = $changeManagementSystemService;
    }

    public function handleManagementSystem($wConsorsFinanz, $shop)
    {
        $success = $this->bootstrapShop($shop);

        if ($success === false) {
            return null;
        }

        $counter = 0;
        $orders = $this->getConsorsOrders($wConsorsFinanz, $shop['id']);
        $wConsorsFinanzPaymentIds = [$wConsorsFinanz['invoicePaymentMethodId'], $wConsorsFinanz['debitPaymentMethodId']];

//        $this->dumb($orders);

        if (empty($orders)) {
            return $counter;
        }

        foreach ($orders as $order) {
            if (!in_array($order['payment']['id'], $wConsorsFinanzPaymentIds)) {
                continue;
            }

            $orderAttribute = $order['attribute'];
            $payment = $order['payment'];

            switch ($orderAttribute['consorsFinanzOrderInvoiceType']) {
                case 'RG':
                case 'TR':
                    if (true == (bool)$orderAttribute['consorsFinanzOrderChangeFlag']) {
                        // Buchung
                        $orderChangeResponse = $this->changeManagementSystemService->change($order, SettingsTable::CHANGE_TYPE_BUCHUNG_BONICHECK);
                        if (0 != $orderChangeResponse->getFehlergrund()) {
                            $this->loggerService->error($orderChangeResponse->getFehlertext() . ' (' . $order['number'] . ')');
                        } else {
                            $this->loggerService->error(sprintf('Order %s successfully changed', $order['number']));
                            // here book
                            $this->dataPersister->persist(['consors_finanz_order_change_flag' => false], 's_order_attributes', $order['id']);
                            ++$counter;
                        }
                    }
                break;
                case 'GU':
                    if (true == (bool)$orderAttribute['consorsFinanzOrderChangeFlag']) {
                        // Gutschrift
                        $orderChangeResponse = $this->changeManagementSystemService->change($order, SettingsTable::CHANGE_TYPE_BUCHUNG_BONICHECK, true);
                        if (0 != $orderChangeResponse->getFehlergrund()) {
                            $this->loggerService->error($orderChangeResponse->getFehlertext() . ' (' . $order['number'] . ')');
                        } else {
                            $this->loggerService->error(sprintf('Order %s successfully changed', $order['number']));

                            $this->dataPersister->persist(['consors_finanz_order_change_flag' => false], 's_order_attributes', $order['id']);
                            ++$counter;
                        }
                    }
                break;
                case 'ST':
                    if (true == (bool)$orderAttribute['consorsFinanzOrderChangeFlag']) {
                        // If the oder has been cancelled, we cancel the order2 as well
                        if ($payment['id'] == $wConsorsFinanz['invoicePaymentMethodId']) {
                            $requestType = SettingsTable::REQUEST_TYPE_INVOICE;
                        } else {
                            $requestType = SettingsTable::REQUEST_TYPE_DEBIT;
                        }

                        $orderStornoResponse = $this->orderStornoService->cancel(
                            $order['number'],
                            $requestType,
                            $orderAttribute['consorsFinanzOrderInvoiceReference']
                        );

                        if (0 != $orderStornoResponse->getFehlergrund()) {
                            $this->loggerService->error($orderStornoResponse->getFehlertext() . ' (' . $order['number'] . ')');
                        } else {
                            $this->loggerService->error(sprintf('Order %s successfully cancelled', $order['number']));
                            $this->dataPersister->persist(['consors_finanz_order_change_flag' => false], 's_order_attributes', $order['id']);
                            ++$counter;
                        }
                    }
                break;
            }
        }

        return $counter;
    }

    public function handleShopwareSystem($wConsorsFinanz, $shop)
    {
        $success = $this->bootstrapShop($shop);

        if ($success === false) {
            return null;
        }

        $counter = 0;
        $wConsorsFinanzPaymentIds = [$wConsorsFinanz['invoicePaymentMethodId'], $wConsorsFinanz['debitPaymentMethodId']];
        $documents = $this->getOrderDocuments($wConsorsFinanz, $shop['id']);

        if (empty($documents)) {
            return $counter;
        }

        foreach ($documents as $document) {
            $documentType = $document['type'];
            $order = $document['order'];
            $payment = $document['order']['payment'];
            if (!in_array($payment['id'], $wConsorsFinanzPaymentIds)) {
                continue;
            }

            $orderAttribute = $document['order']['attribute'];
            switch ($documentType['name']) {
                case self::INVOICE: {
                    if ($document['amount'] == $order['invoiceAmount']) {
                        // if not booked
                        if (false == (bool)$orderAttribute['consorsFinanzIsOrderBooked']) {
                            // Buchung
                            $orderChangeResponse = $this->changeService->change($document, SettingsTable::CHANGE_TYPE_BUCHUNG_BONICHECK);
                            if (0 != $orderChangeResponse->getFehlergrund()) {
                                $this->loggerService->error($orderChangeResponse->getFehlertext() . ' (' . $order['number'] . ')');
                            } else {
                                $this->loggerService->error(sprintf('Order %s successfully changed', $order['number']));
                                // here book
                                $this->dataPersister->persist(['consors_finanz_is_order_booked' => true], 's_order_attributes', $order['id']);
                                ++$counter;
                            }
                        }
                    }
                }
                break;
                case self::CANCELLATION: {
                    // if not yet cancelled
                    if (false == (bool)$orderAttribute['consorsFinanzIsOrderCancelled']) {
                        // If the oder has been cancelled, we cancel the order2 as well
                        if ($payment['id'] == $wConsorsFinanz['invoicePaymentMethodId']) {
                            $requestType = SettingsTable::REQUEST_TYPE_INVOICE;
                        } else {
                            $requestType = SettingsTable::REQUEST_TYPE_DEBIT;
                        }

                        $orderStornoResponse = $this->orderStornoService->cancel($order['number'], $requestType);
                        if (0 != $orderStornoResponse->getFehlergrund()) {
                            $this->loggerService->error($orderStornoResponse->getFehlertext() . ' (' . $order['number'] . ')');
                        } else {
                            $this->loggerService->error(sprintf('Order %s successfully cancelled', $order['number']));
                            $this->dataPersister->persist(['consors_finanz_is_order_cancelled' => true], 's_order_attributes', $order['id']);
                            ++$counter;
                        }
                    }
                }
                break;
                case self::GUTSCHRIFT: {
                    $creditNumbers = $orderAttribute['consorsFinanzOrderCreditNumbers'];
                    $arrayCreditNumbers = explode(',', $creditNumbers);
                    if (!in_array($document['documentId'], $arrayCreditNumbers)) {
                        // Gutschrift
                        $orderChangeResponse = $this->changeService->change($document, SettingsTable::CHANGE_TYPE_BUCHUNG_BONICHECK, true);
                        if (0 != $orderChangeResponse->getFehlergrund()) {
                            $this->loggerService->error($orderChangeResponse->getFehlertext() . ' (' . $order['number'] . ')');
                        } else {
                            $this->loggerService->error(sprintf('Order %s successfully changed', $order['number']));

                            $newNumbers = '' == $creditNumbers ? $document['documentId'] : $creditNumbers . ',' . $document['documentId'];
                            $this->dataPersister->persist(['consors_finanz_order_credit_numbers' => $newNumbers],
                                's_order_attributes', $order['id']);
                            ++$counter;
                        }
                    }
                }
                break;
            }
        }

        return $counter;
    }

    /**
     * @param $wConsorsFinanz
     * @param $shopId
     * @return array
     */
    private function getConsorsOrders($wConsorsFinanz, $shopId)
    {
        $excludeOrderIds = [$wConsorsFinanz['completedOrderState']];
        $wConsorsFinanzPaymentIds = [$wConsorsFinanz['invoicePaymentMethodId'], $wConsorsFinanz['debitPaymentMethodId']];

        $orders = $this->modelManager->getRepository(Order::class)->createQueryBuilder('swOrder')
            ->select('swOrder', 'attribute', 'details', 'customer', 'billing', 'shipping', 'payment')
            ->leftJoin('swOrder.attribute', 'attribute')
            ->leftJoin('swOrder.details', 'details')
            ->leftJoin('swOrder.customer', 'customer')
            ->leftJoin('swOrder.billing', 'billing')
            ->leftJoin('swOrder.shipping', 'shipping')
            ->leftJoin('swOrder.payment', 'payment')
            ->where('swOrder.status NOT IN (:excludeOrderIds)')
            ->andWhere('swOrder.paymentId IN (:wConsorsFinanzPaymentIds)')
            ->andWhere('attribute.consorsFinanzOrderChangeFlag = 1')
            ->andWhere('swOrder.shopId = :shopId')
            ->setParameter('excludeOrderIds', $excludeOrderIds)
            ->setParameter('wConsorsFinanzPaymentIds', $wConsorsFinanzPaymentIds)
            ->setParameter('shopId', $shopId)
            ->addOrderBy('swOrder.id', 'DESC')
            ->getQuery()
            ->getArrayResult()
        ;

        if ($orders) {
            foreach ($orders as &$order) {
                $order['invoiceAmount'] = round($order['order']['invoiceAmount'] * 100, 2);
                $order['invoiceAmountNet'] = round($order['order']['invoiceAmountNet'] * 100, 2);
                $order['invoiceShipping'] = round($order['order']['invoiceShipping'] * 100, 2);
                $order['invoiceShippingNet'] = round($order['order']['invoiceShippingNet'] * 100, 2);

                foreach ($order['details'] as &$detail) {
                    $detail['price'] = round($detail['price'] * 100, 2);
                }
            }
        }

        return $orders;
    }

    /**
     * @param $wConsorsFinanz
     * @param $shopId
     * @return array
     */
    private function getOrderDocuments($wConsorsFinanz, $shopId)
    {
        $excludeOrderIds = [$wConsorsFinanz['completedOrderStatus'], Status::ORDER_STATE_CANCELLED];
        $wConsorsFinanzPaymentIds = [$wConsorsFinanz['invoicePaymentMethodId'], $wConsorsFinanz['debitPaymentMethodId']];

        $documents = $this->modelManager->getRepository(Document::class)->createQueryBuilder('document')
            ->select('document', 'documentType', 'swOrder', 'attribute', 'details', 'customer', 'billing', 'shipping', 'payment')
            ->leftJoin('document.type', 'documentType')
            ->leftJoin('document.order', 'swOrder')
            ->leftJoin('swOrder.attribute', 'attribute')
            ->leftJoin('swOrder.details', 'details')
            ->leftJoin('swOrder.customer', 'customer')
            ->leftJoin('swOrder.billing', 'billing')
            ->leftJoin('swOrder.shipping', 'shipping')
            ->leftJoin('swOrder.payment', 'payment')
            ->where('swOrder.status NOT IN (:excludeOrderIds)')
            ->andWhere('swOrder.paymentId IN (:wConsorsFinanzPaymentIds)')
            ->andWhere('swOrder.shopId = :shopId')
            ->setParameter('excludeOrderIds', $excludeOrderIds)
            ->setParameter('wConsorsFinanzPaymentIds', $wConsorsFinanzPaymentIds)
            ->setParameter('shopId', $shopId)
            ->addOrderBy('document.id', 'DESC')
            ->getQuery()
            ->getArrayResult()
        ;

        if ($documents) {
            foreach ($documents as &$document) {
                $document['amount'] = round($document['amount'] * 100, 2);
                $document['order']['invoiceAmount'] = round($document['order']['invoiceAmount'] * 100, 2);
                $document['order']['invoiceAmountNet'] = round($document['order']['invoiceAmountNet'] * 100, 2);
                $document['order']['invoiceShipping'] = round($document['order']['invoiceShipping'] * 100, 2);
                $document['order']['invoiceShippingNet'] = round($document['order']['invoiceShippingNet'] * 100, 2);

                foreach ($document['order']['details'] as &$detail) {
                    $detail['price'] = round($detail['price'] * 100, 2);
                }
            }
        }

        return $documents;
    }

    /**
     * @param $shop
     *
     * @return bool
     */
    private function bootstrapShop($shop)
    {
        /* @var Shop $shopObject */
        $shopObject = $this->modelManager->getRepository(Shop::class)->find($shop['id']);

        if (!$shopObject) {
            return false;
        }

        $shopObject->registerResources();

        return true;
    }

    function dumb($data)
    {
        highlight_string("<?php\n " . var_export($data, true) . "?>");
        echo '<script>document.getElementsByTagName("code")[0].getElementsByTagName("span")[1].remove() ;document.getElementsByTagName("code")[0].getElementsByTagName("span")[document.getElementsByTagName("code")[0].getElementsByTagName("span").length - 1].remove() ; </script>';
        die();
    }
}